Package: ZetaSuite
Type: Package
Title: Analyze High-Dimensional High-Throughput Dataset and Quality
        Control Single-Cell RNA-Seq
Version: 1.0.3
Date: 2026-04-27
Authors@R: c(
    person(given = "Junhui", family = "Li", email = "ljh.biostat@gmail.com", role = c("aut","cre"), comment=c(ORCID="0000-0003-3973-1700")),
    person(given = "Yajing", family = "Hao", email = "yahao@health.ucsd.edu", role  = c("aut"), comment = c(ORCID = "0000-0003-1384-4176")),
    person(given = "Shuyang", family = "Zhang", email = "s4zhang@ucsd.edu", role  = "ctb", comment=c(ORCID="0000-0002-8428-1828")),
    person(given = "Guofeng", family = "Zhao", email = "gzhao@health.ucsd.edu", role = "ctb"),
    person(given = "Xiang-Dong", family = "Fu", email = "xdfu@health.ucsd.edu", role = c("cph","fnd"), comment = c(ORCID = "0000-0001-5499-8732"))
		)
Maintainer: Junhui Li <ljh.biostat@gmail.com>
Description: The advent of genomic technologies has enabled the generation of two-dimensional or even multi-dimensional high-throughput data, e.g., monitoring multiple changes in gene expression in genome-wide siRNA screens across many different cell types (E Robert McDonald 3rd (2017) <doi: 10.1016/j.cell.2017.07.005> and Tsherniak A (2017) <doi: 10.1016/j.cell.2017.06.010>) or single cell transcriptomics under different experimental conditions. We found that simple computational methods based on a single statistical criterion is no longer adequate for analyzing such multi-dimensional data. We herein introduce 'ZetaSuite', a statistical package initially designed to score hits from two-dimensional RNAi screens.We also illustrate a unique utility of 'ZetaSuite' in analyzing single cell transcriptomics to differentiate rare cells from damaged ones (Vento-Tormo R (2018) <doi: 10.1038/s41586-018-0698-6>). In 'ZetaSuite', we  have the following steps: QC of input datasets, normalization using Z-transformation, Zeta score calculation and hits selection based on defined Screen Strength.
BugReports: https://github.com/JunhuiLi1017/ZetaSuite/issues
Imports: RColorBrewer, Rtsne, e1071, ggplot2, reshape2, gridExtra,
        mixtools, shinyjs, shinydashboard, shiny, plotly, DT
License: MIT + file LICENSE
Depends: R (>= 2.10)
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-28 01:30:39 UTC; lij11
Repository: CRAN
Date/Publication: 2026-04-28 07:00:02 UTC
Encoding: UTF-8
Author: Junhui Li [aut, cre] (ORCID: <https://orcid.org/0000-0003-3973-1700>),
  Yajing Hao [aut] (ORCID: <https://orcid.org/0000-0003-1384-4176>),
  Shuyang Zhang [ctb] (ORCID: <https://orcid.org/0000-0002-8428-1828>),
  Guofeng Zhao [ctb],
  Xiang-Dong Fu [cph, fnd] (ORCID:
    <https://orcid.org/0000-0001-5499-8732>)
Built: R 4.6.0; ; 2026-04-28 11:15:42 UTC; unix

# Generated by roxygen2: do not edit by hand

export(EventCoverage)
export(FDRcutoff)
export(QC)
export(SVM)
export(Zeta)
export(ZetaSuitSC)
export(ZetaSuiteApp)
export(Zscore)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,renderDataTable)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rtsne,Rtsne)
importFrom(e1071,svm)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(mixtools,normalmixEM)
importFrom(plotly,ggplotly)
importFrom(reshape2,melt)
importFrom(shiny,a)
importFrom(shiny,actionButton)
importFrom(shiny,br)
importFrom(shiny,checkboxInput)
importFrom(shiny,code)
importFrom(shiny,downloadButton)
importFrom(shiny,fileInput)
importFrom(shiny,fluidRow)
importFrom(shiny,h3)
importFrom(shiny,h4)
importFrom(shiny,numericInput)
importFrom(shiny,observeEvent)
importFrom(shiny,p)
importFrom(shiny,plotOutput)
importFrom(shiny,reactiveValues)
importFrom(shiny,renderPlot)
importFrom(shiny,req)
importFrom(shiny,runApp)
importFrom(shiny,showNotification)
importFrom(shiny,tabPanel)
importFrom(shiny,tabsetPanel)
importFrom(shiny,tags)
importFrom(shiny,textOutput)
importFrom(shiny,verbatimTextOutput)
importFrom(shinydashboard,box)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardPage)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinydashboard,menuItem)
importFrom(shinydashboard,sidebarMenu)
importFrom(shinydashboard,tabItem)
importFrom(shinydashboard,tabItems)
importFrom(shinyjs,useShinyjs)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,var)

# ZetaSuite

An R package for analyzing multi-dimensional high-throughput screening data, particularly two-dimensional RNAi screens and single-cell RNA sequencing data.

## Installation

```r
# Install from CRAN
install.packages("ZetaSuite")

# Or install from GitHub
devtools::install_github("username/ZetaSuite")

# Load the package
library(ZetaSuite)
```

## Quick Start

```r
# Load example data
data(countMat)
data(negGene)
data(posGene)
data(nonExpGene)

# Quality Control
qc_results <- QC(countMat, negGene, posGene)

# Z-score normalization
zscore_matrix <- Zscore(countMat, negGene)

# Event coverage analysis
ec_results <- EventCoverage(zscore_matrix, negGene, posGene)

# Zeta score calculation
zeta_scores <- Zeta(zscore_matrix, ec_results[[1]]$ZseqList)

# FDR cutoff analysis
fdr_results <- FDRcutoff(zeta_scores, negGene, posGene, nonExpGene)
```

## Interactive Shiny Application

Launch the interactive web interface for ZetaSuite:

```r
# Launch the Shiny app
ZetaSuiteApp()

# Launch without opening browser automatically
ZetaSuiteApp(launch.browser = FALSE)

# Launch on a specific port
ZetaSuiteApp(port = 3838)
```

The Shiny app provides:
- Interactive data upload and visualization
- Step-by-step analysis workflow
- Real-time results and plots
- Data export capabilities
- Built-in example dataset

## Features

- **Quality Control Analysis**: Comprehensive evaluation of experimental design and data quality
- **Z-score Normalization**: Standardization using negative controls as reference
- **Event Coverage Analysis**: Quantification of regulatory effects across thresholds
- **Zeta Score Calculation**: Area-under-curve based scoring for regulatory effects
- **SVM-based Background Correction**: Machine learning approach to filter noise
- **Screen Strength Analysis**: Optimal threshold selection for hit identification
- **Single Cell Quality Control**: Cell quality assessment for scRNA-seq data

## Documentation

For detailed documentation and examples, see the package vignette:

```r
vignette("ZetaSuite")
```

## Bug Reports

If you encounter any bugs or have feature requests, please report them on our GitHub issues page:

[Report a Bug](https://github.com/JunhuiLi1017/ZetaSuite/issues)

## Citation

If you use ZetaSuite in your research, please cite:

Hao, Y., Zhang, S., Shao, C. et al. ZetaSuite: computational analysis of two-dimensional high-throughput data from multi-target screens and single-cell transcriptomics. Genome Biol 23, 162 (2022). https://doi.org/10.1186/s13059-022-02729-4

## License

This package is licensed under the MIT License - see the LICENSE file for details.

# ZetaSuite Shiny Application

A comprehensive web-based interface for the ZetaSuite R package, providing an intuitive way to analyze high-throughput screening data and perform single-cell RNA-seq quality control.

## Features

- **Interactive Web Interface**: User-friendly dashboard with tabbed navigation
- **Built-in Example Dataset**: Explore the HTS2 screening dataset included with the package
- **Complete Analysis Workflow**: Step-by-step analysis from QC to hit selection
- **Interactive Visualizations**: Plotly-powered interactive plots with zoom, pan, and hover
- **Data Upload Support**: Upload your own CSV files for analysis
- **Results Download**: Export all analysis results as CSV files
- **Real-time Progress**: Progress indicators and status updates
- **Error Handling**: Comprehensive error messages and notifications

## Analysis Modules

### 1. Quality Control Analysis
- Score distribution plots
- t-SNE visualization for sample separation
- Box plots comparing control groups
- SSMD (Strictly Standardized Mean Difference) distribution

### 2. Z-score Normalization
- Standardization using negative controls
- Interactive data table preview
- Download normalized results

### 3. Event Coverage Analysis
- Quantification of regulatory effects across thresholds
- Separate analysis for decrease and increase directions
- Interactive jitter plots

### 4. Zeta Score Calculation
- Area-under-curve based scoring
- Optional SVM background correction
- Top hits identification by direction

### 5. SVM Analysis
- Machine learning-based decision boundaries
- Background correction for improved accuracy
- Separate analysis for decrease and increase directions

### 6. FDR Cutoff Analysis
- Screen Strength calculation
- Optimal threshold selection
- Interactive hit selection with customizable thresholds

### 7. Single Cell Quality Control
- Cell quality assessment for scRNA-seq data
- Gaussian mixture model-based cutoff determination
- Quality score distribution visualization

## Installation

### Prerequisites

Make sure you have R installed (version 3.6 or higher) and the following packages:

```r
# Install required packages
install.packages(c("shiny", "shinydashboard", "DT", "plotly", "shinyjs"))

# Install ZetaSuite package dependencies
install.packages(c("RColorBrewer", "Rtsne", "e1071", "ggplot2", "reshape2", 
                   "gridExtra", "mixtools"))
```

### Setup

1. Install the ZetaSuite package
2. Place the `app.R` file in your working directory
3. Run the Shiny application

## Usage

### Starting the Application

```r
# Load the ZetaSuite package
library(ZetaSuite)

# Run the Shiny app
shiny::runApp("app.R")
```

### Quick Start with Example Data

1. **Load Example Data**: Click the "Example Data" tab and press "Load Example Data"
2. **Quality Control**: Go to the "Quality Control" tab and run QC analysis
3. **Z-score Analysis**: Calculate normalized Z-scores
4. **Event Coverage**: Generate event coverage analysis
5. **Zeta Score**: Calculate regulatory effect scores
6. **FDR Cutoff**: Determine optimal thresholds
7. **Hit Selection**: Select significant hits based on Screen Strength
8. **Download Results**: Export all analysis results

### Using Your Own Data

#### Data Format Requirements

1. **Count Matrix (CSV)**:
   - Rows: Genes/siRNAs
   - Columns: Readouts/conditions
   - First column should contain gene/siRNA identifiers
   - Example:
     ```
     Gene,Readout1,Readout2,Readout3
     Gene1,10.5,12.3,8.9
     Gene2,15.2,14.1,16.7
     ```

2. **Negative Control Genes (CSV)**:
   - First column: Gene/siRNA identifiers matching count matrix
   - Example:
     ```
     Gene
     NegCtrl1
     NegCtrl2
     NegCtrl3
     ```

3. **Positive Control Genes (CSV)**:
   - First column: Gene/siRNA identifiers matching count matrix
   - Example:
     ```
     Gene
     PosCtrl1
     PosCtrl2
     PosCtrl3
     ```

4. **Non-expressed Genes (CSV, optional)**:
   - First column: Gene/siRNA identifiers matching count matrix
   - Used for FDR cutoff analysis

#### Single Cell Data

**Single Cell Count Matrix (CSV)**:
- Rows: Cells
- Columns: Genes
- First column should contain cell identifiers
- Example:
  ```
  Cell,Gene1,Gene2,Gene3
  Cell1,5,0,12
  Cell2,0,8,3
  Cell3,15,2,0
  ```

### Analysis Workflow

1. **Data Upload**: Upload your data files in the "Data Upload" tab
2. **Quality Control**: Run QC analysis to assess data quality
3. **Z-score Analysis**: Normalize your data using negative controls
4. **Event Coverage**: Calculate event coverage across thresholds
5. **Zeta Score**: Compute regulatory effect scores
6. **SVM Analysis** (optional): Generate decision boundaries
7. **FDR Cutoff**: Determine optimal thresholds
8. **Hit Selection**: Select significant hits based on Screen Strength
9. **Single Cell QC** (if applicable): Quality control for single-cell data
10. **Results**: Download all analysis results

### Parameter Settings

#### Event Coverage Parameters
- **Number of Bins**: Number of Z-score thresholds (default: 100)
- **Combine Directions**: Whether to combine decrease and increase directions

#### Zeta Score Parameters
- **Use SVM Curves**: Whether to use SVM curves for background correction

#### FDR Cutoff Parameters
- **Combine Directions**: Whether to combine decrease and increase directions
- **Screen Strength Threshold**: Minimum SS value for hit selection (default: 0.8)

#### Single Cell QC Parameters
- **Number of Bins**: Number of expression thresholds (default: 10)
- **Filter Low Count Cells**: Remove cells with <100 total reads

## Example Dataset

The application includes the HTS2 screening dataset from the ZetaSuite package:

- **1,609 genes × 100 alternative splicing events**
- **30 negative control genes** (non-specific siRNAs)
- **20 positive control genes** (PTB-targeting siRNAs)
- **50 non-expressed genes** (RPKM < 1 in HeLa cells)

This dataset demonstrates the complete analysis workflow and can be used to explore the application's features.

## Output Files

The application generates several output files:

1. **Z-score Results**: Normalized Z-score matrix
2. **Zeta Scores**: Regulatory effect scores for each gene
3. **FDR Results**: Cutoff thresholds and Screen Strength values
4. **Selected Hits**: Genes identified as significant based on Screen Strength
5. **Single Cell Results**: Cell quality scores and cutoff values

## Interactive Features

- **Data Tables**: Sortable and searchable data tables with horizontal scrolling
- **Interactive Plots**: Zoom, pan, and hover over plotly visualizations
- **Progress Indicators**: Real-time progress updates for long-running analyses
- **Error Handling**: Comprehensive error messages and notifications
- **Status Updates**: Real-time status updates for each analysis step

## Troubleshooting

### Common Issues

1. **Package Not Found**: Make sure all required packages are installed
2. **Data Format Errors**: Ensure CSV files have correct column headers and data types
3. **Memory Issues**: For large datasets, consider reducing the number of bins or filtering data
4. **SVM Analysis Fails**: Ensure you have sufficient positive and negative control samples

### Performance Tips

- For large datasets (>10,000 genes), consider preprocessing to reduce computational time
- Use the "Filter Low Count Cells" option for single-cell data to remove low-quality cells
- Adjust the number of bins based on your data size and computational resources
- Use `combine = TRUE` for faster event coverage analysis

### Error Messages

- **"Error loading data"**: Check file format and ensure all required files are uploaded
- **"Error in Quality Control"**: Verify that positive and negative control genes are present in the count matrix
- **"Error in Z-score calculation"**: Ensure negative control genes are properly formatted
- **"Error in Event Coverage"**: Check that Z-scores have been calculated first

## Advanced Usage

### Custom Analysis Workflows

The application supports custom analysis workflows:

1. **SVM Background Correction**: Enable SVM analysis for improved accuracy
2. **Separate Direction Analysis**: Analyze decrease and increase directions separately
3. **Custom Thresholds**: Adjust Screen Strength thresholds for different sensitivity/specificity trade-offs
4. **Batch Processing**: Process multiple datasets by uploading different files

### Integration with R Scripts

The Shiny app can be integrated with custom R scripts:

```r
# Load results from Shiny app
zscore_results <- read.csv("zscore_results.csv", row.names = 1)
zeta_results <- read.csv("zeta_scores.csv", row.names = 1)

# Continue with custom analysis
# ...
```

## Support

For issues related to the ZetaSuite package functionality, please refer to the main package documentation. For Shiny app-specific issues:

1. Check the R console for error messages
2. Ensure all dependencies are properly installed
3. Verify data format requirements
4. Check the troubleshooting section above

## References

- **Software**: Hao, Y., Shao, C., Zhao, G., Fu, X.D. (2021). ZetaSuite: A Computational Method for Analyzing Multi-dimensional High-throughput Data, Reveals Genes with Opposite Roles in Cancer Dependency. *Forthcoming*

- **Dataset**: Shao, C., Hao, Y., Qiu, J., Zhou, B., Li, H., Zhou, Y., Meng, F., Jiang, L., Gou, L.T., Xu, J., Li, Y., Wang, H., Yeo, G.W., Wang, D., Ji, X., Glass, C.K., Aza-Blanc, P., Fu, X.D. (2021). HTS2 Screen for Global Splicing Regulators Reveals a Key Role of the Pol II Subunit RPB9 in Coupling between Transcription and Pre-mRNA Splicing. *Cell. Forthcoming*

## License

This Shiny application is provided under the same license as the ZetaSuite package (MIT + file LICENSE). 

# Install Dependencies for ZetaSuite Shiny App
# This script installs all required packages for the Shiny application

# Function to check if a package is installed
is_package_installed <- function(package_name) {
  return(package_name %in% installed.packages()[,"Package"])
}

# Function to install package if not already installed
install_if_missing <- function(package_name, source = "CRAN") {
  if (!is_package_installed(package_name)) {
    cat("Installing", package_name, "...\n")
    if (source == "CRAN") {
      install.packages(package_name, dependencies = TRUE)
    } else if (source == "BiocManager") {
      if (!requireNamespace("BiocManager", quietly = TRUE)) {
        install.packages("BiocManager")
      }
      BiocManager::install(package_name)
    }
    cat("✓", package_name, "installed successfully\n")
  } else {
    cat("✓", package_name, "already installed\n")
  }
}

# Main installation function
install_shiny_dependencies <- function() {
  cat("Installing dependencies for ZetaSuite Shiny App...\n\n")
  
  # Core Shiny packages
  cat("1. Installing core Shiny packages...\n")
  shiny_packages <- c("shiny", "shinydashboard", "DT", "plotly", "shinyjs")
  for (pkg in shiny_packages) {
    install_if_missing(pkg)
  }
  
  # ZetaSuite package dependencies
  cat("\n2. Installing ZetaSuite package dependencies...\n")
  zetasuite_packages <- c(
    "RColorBrewer",
    "Rtsne", 
    "e1071",
    "ggplot2",
    "reshape2",
    "gridExtra",
    "mixtools"
  )
  for (pkg in zetasuite_packages) {
    install_if_missing(pkg)
  }
  
  # Additional useful packages
  cat("\n3. Installing additional useful packages...\n")
  additional_packages <- c(
    "dplyr",
    "tidyr",
    "readr",
    "writexl"
  )
  for (pkg in additional_packages) {
    install_if_missing(pkg)
  }
  
  cat("\n✓ All dependencies installed successfully!\n")
  cat("\nYou can now run the ZetaSuite Shiny app using:\n")
  cat("  shiny::runApp('app.R')\n")
}

# Function to check if all dependencies are available
check_dependencies <- function() {
  cat("Checking ZetaSuite Shiny App dependencies...\n\n")
  
  required_packages <- c(
    "shiny", "shinydashboard", "DT", "plotly", "shinyjs",
    "RColorBrewer", "Rtsne", "e1071", "ggplot2", "reshape2", 
    "gridExtra", "mixtools", "dplyr", "tidyr", "readr", "writexl"
  )
  
  missing_packages <- c()
  installed_packages <- c()
  
  for (pkg in required_packages) {
    if (is_package_installed(pkg)) {
      installed_packages <- c(installed_packages, pkg)
      cat("✓", pkg, "\n")
    } else {
      missing_packages <- c(missing_packages, pkg)
      cat("✗", pkg, "(missing)\n")
    }
  }
  
  cat("\nSummary:\n")
  cat("- Installed packages:", length(installed_packages), "/", length(required_packages), "\n")
  cat("- Missing packages:", length(missing_packages), "/", length(required_packages), "\n")
  
  if (length(missing_packages) > 0) {
    cat("\nMissing packages:\n")
    for (pkg in missing_packages) {
      cat("  -", pkg, "\n")
    }
    cat("\nRun install_shiny_dependencies() to install missing packages.\n")
    return(FALSE)
  } else {
    cat("\n✓ All dependencies are installed and ready!\n")
    return(TRUE)
  }
}

# Function to load all required packages
load_dependencies <- function() {
  cat("Loading ZetaSuite Shiny App dependencies...\n\n")
  
  required_packages <- c(
    "shiny", "shinydashboard", "DT", "plotly", "shinyjs",
    "RColorBrewer", "Rtsne", "e1071", "ggplot2", "reshape2", 
    "gridExtra", "mixtools", "dplyr", "tidyr", "readr", "writexl"
  )
  
  failed_packages <- c()
  loaded_packages <- c()
  
  for (pkg in required_packages) {
    tryCatch({
      library(pkg, character.only = TRUE)
      loaded_packages <- c(loaded_packages, pkg)
      cat("✓", pkg, "loaded\n")
    }, error = function(e) {
      failed_packages <- c(failed_packages, pkg)
      cat("✗", pkg, "failed to load\n")
    })
  }
  
  cat("\nSummary:\n")
  cat("- Successfully loaded:", length(loaded_packages), "/", length(required_packages), "\n")
  cat("- Failed to load:", length(failed_packages), "/", length(required_packages), "\n")
  
  if (length(failed_packages) > 0) {
    cat("\nFailed packages:\n")
    for (pkg in failed_packages) {
      cat("  -", pkg, "\n")
    }
    cat("\nSome packages may need to be reinstalled.\n")
    return(FALSE)
  } else {
    cat("\n✓ All dependencies loaded successfully!\n")
    return(TRUE)
  }
}

# Function to run the Shiny app
run_shiny_app <- function() {
  cat("Starting ZetaSuite Shiny App...\n")
  
  # Check if app.R exists
  if (!file.exists("app.R")) {
    cat("Error: app.R file not found in current directory.\n")
    cat("Please make sure you're in the correct directory.\n")
    return(FALSE)
  }
  
  # Check dependencies
  if (!check_dependencies()) {
    cat("Please install missing dependencies first.\n")
    return(FALSE)
  }
  
  # Load dependencies
  if (!load_dependencies()) {
    cat("Failed to load some dependencies.\n")
    return(FALSE)
  }
  
  # Run the app
  cat("\nLaunching Shiny app...\n")
  cat("The app will open in your default web browser.\n")
  cat("To stop the app, press Ctrl+C in the R console.\n\n")
  
  tryCatch({
    shiny::runApp("app.R", launch.browser = TRUE)
  }, error = function(e) {
    cat("Error launching Shiny app:", e$message, "\n")
    return(FALSE)
  })
}

# Main execution
if (interactive()) {
  cat("ZetaSuite Shiny App Setup\n")
  cat("========================\n\n")
  cat("Available functions:\n")
  cat("1. install_shiny_dependencies() - Install all required packages\n")
  cat("2. check_dependencies() - Check if all packages are installed\n")
  cat("3. load_dependencies() - Load all required packages\n")
  cat("4. run_shiny_app() - Start the Shiny application\n")
  cat("5. generate_example_data() - Generate test data (from generate_example_data.R)\n")
  cat("\nTo get started, run: install_shiny_dependencies()\n")
} else {
  cat("To use this script interactively, run it in R console.\n")
} 

# Generate Example Data for ZetaSuite Shiny App
# This script creates sample data files that can be used to test the application

# Set seed for reproducibility
set.seed(123)

# Generate example count matrix
generate_count_matrix <- function(n_genes = 1000, n_samples = 50) {
  # Create gene names
  gene_names <- paste0("Gene", 1:n_genes)
  
  # Generate count data with some structure
  # Most genes have moderate expression
  base_expression <- rnorm(n_genes, mean = 10, sd = 3)
  
  # Create sample-specific effects
  sample_effects <- rnorm(n_samples, mean = 0, sd = 2)
  
  # Generate count matrix
  count_matrix <- matrix(0, nrow = n_genes, ncol = n_samples)
  
  for (i in 1:n_genes) {
    for (j in 1:n_samples) {
      # Add some noise and ensure positive values
      count_matrix[i, j] <- max(0, base_expression[i] + sample_effects[j] + rnorm(1, 0, 1))
    }
  }
  
  # Add some genes with high expression (potential positive controls)
  high_expr_genes <- sample(1:n_genes, 20)
  for (i in high_expr_genes) {
    count_matrix[i, ] <- count_matrix[i, ] * runif(1, 2, 5)
  }
  
  # Add some genes with low expression (potential negative controls)
  low_expr_genes <- sample(setdiff(1:n_genes, high_expr_genes), 30)
  for (i in low_expr_genes) {
    count_matrix[i, ] <- count_matrix[i, ] * runif(1, 0.1, 0.5)
  }
  
  # Set row and column names
  rownames(count_matrix) <- gene_names
  colnames(count_matrix) <- paste0("Sample", 1:n_samples)
  
  return(count_matrix)
}

# Generate control gene lists
generate_control_genes <- function(count_matrix, n_neg = 30, n_pos = 20, n_non_exp = 50) {
  gene_names <- rownames(count_matrix)
  
  # Calculate mean expression for each gene
  mean_expr <- rowMeans(count_matrix)
  
  # Select negative controls (low expression)
  neg_genes <- names(sort(mean_expr))[1:n_neg]
  
  # Select positive controls (high expression)
  pos_genes <- names(sort(mean_expr, decreasing = TRUE))[1:n_pos]
  
  # Select non-expressed genes (very low expression)
  non_exp_genes <- names(sort(mean_expr))[1:n_non_exp]
  
  return(list(
    neg_genes = data.frame(Gene = neg_genes, stringsAsFactors = FALSE),
    pos_genes = data.frame(Gene = pos_genes, stringsAsFactors = FALSE),
    non_exp_genes = data.frame(Gene = non_exp_genes, stringsAsFactors = FALSE)
  ))
}

# Generate single cell count matrix
generate_single_cell_data <- function(n_cells = 500, n_genes = 2000) {
  # Create cell and gene names
  cell_names <- paste0("Cell", 1:n_cells)
  gene_names <- paste0("Gene", 1:n_genes)
  
  # Generate sparse count matrix (typical for single-cell data)
  count_matrix <- matrix(0, nrow = n_cells, ncol = n_genes)
  
  # Set row and column names
  rownames(count_matrix) <- cell_names
  colnames(count_matrix) <- gene_names
  
  # Generate expression for each cell-gene combination
  for (i in 1:n_cells) {
    # Each cell expresses a subset of genes
    n_expressed <- rpois(1, lambda = 500)  # Average 500 genes per cell
    expressed_genes <- sample(1:n_genes, min(n_expressed, n_genes))
    
    for (j in expressed_genes) {
      # Generate count from negative binomial distribution
      count_matrix[i, j] <- rnbinom(1, mu = 5, size = 1)
    }
  }
  
  # Add some high-quality cells (more genes expressed)
  high_quality_cells <- sample(1:n_cells, 50)
  for (i in high_quality_cells) {
    extra_genes <- sample(setdiff(1:n_genes, which(count_matrix[i, ] > 0)), 200)
    for (j in extra_genes) {
      count_matrix[i, j] <- rnbinom(1, mu = 3, size = 1)
    }
  }
  
  # Add some low-quality cells (fewer genes expressed)
  low_quality_cells <- sample(setdiff(1:n_cells, high_quality_cells), 100)
  for (i in low_quality_cells) {
    # Remove some expressed genes
    expressed <- which(count_matrix[i, ] > 0)
    to_remove <- sample(expressed, length(expressed) * 0.7)
    count_matrix[i, to_remove] <- 0
  }
  
  return(count_matrix)
}

# Main function to generate all example data
generate_example_data <- function() {
  cat("Generating example data for ZetaSuite Shiny App...\n")
  
  # Generate main analysis data
  cat("1. Generating count matrix...\n")
  count_matrix <- generate_count_matrix(n_genes = 1000, n_samples = 50)
  
  cat("2. Generating control gene lists...\n")
  controls <- generate_control_genes(count_matrix)
  
  # Generate single cell data
  cat("3. Generating single cell count matrix...\n")
  single_cell_matrix <- generate_single_cell_data(n_cells = 500, n_genes = 2000)
  
  # Write files
  cat("4. Writing data files...\n")
  
  # Write count matrix
  write.csv(count_matrix, "example_count_matrix.csv")
  cat("   - example_count_matrix.csv\n")
  
  # Write control gene lists
  write.csv(controls$neg_genes, "example_negative_controls.csv", row.names = FALSE)
  cat("   - example_negative_controls.csv\n")
  
  write.csv(controls$pos_genes, "example_positive_controls.csv", row.names = FALSE)
  cat("   - example_positive_controls.csv\n")
  
  write.csv(controls$non_exp_genes, "example_non_expressed_genes.csv", row.names = FALSE)
  cat("   - example_non_expressed_genes.csv\n")
  
  # Write single cell matrix
  write.csv(single_cell_matrix, "example_single_cell_matrix.csv")
  cat("   - example_single_cell_matrix.csv\n")
  
  cat("\nExample data generation complete!\n")
  cat("You can now use these files to test the ZetaSuite Shiny application.\n")
  cat("\nFile descriptions:\n")
  cat("- example_count_matrix.csv: Main count matrix for analysis\n")
  cat("- example_negative_controls.csv: Negative control genes\n")
  cat("- example_positive_controls.csv: Positive control genes\n")
  cat("- example_non_expressed_genes.csv: Non-expressed genes for FDR analysis\n")
  cat("- example_single_cell_matrix.csv: Single cell count matrix\n")
  
  # Return summary statistics
  cat("\nData summary:\n")
  cat("- Count matrix:", nrow(count_matrix), "genes ×", ncol(count_matrix), "samples\n")
  cat("- Negative controls:", nrow(controls$neg_genes), "genes\n")
  cat("- Positive controls:", nrow(controls$pos_genes), "genes\n")
  cat("- Non-expressed genes:", nrow(controls$non_exp_genes), "genes\n")
  cat("- Single cell matrix:", nrow(single_cell_matrix), "cells ×", ncol(single_cell_matrix), "genes\n")
}

# Run the data generation
if (interactive()) {
  generate_example_data()
} else {
  cat("To generate example data, run: generate_example_data()\n")
} 

# ZetaSuite Shiny Application
# A web interface for high-throughput screening data analysis

library(shiny)
library(shinydashboard)
library(DT)
library(plotly)
library(shinyjs)

# UI Definition
ui <- dashboardPage(
  dashboardHeader(
    title = tags$a(href = "https://cran.r-project.org/web/packages/ZetaSuite/index.html", 
                   "ZetaSuite", 
                   target = "_blank",
                   style = "color: white; text-decoration: none;")
  ),
  
  dashboardSidebar(
    sidebarMenu(
      menuItem("Welcome", tabName = "welcome", icon = icon("home")),
      menuItem("Data Upload", tabName = "upload", icon = icon("upload")),
      menuItem("Example Data", tabName = "example", icon = icon("database")),
      menuItem("Quality Control", tabName = "qc", icon = icon("check-circle")),
      menuItem("Z-score Analysis", tabName = "zscore", icon = icon("chart-line")),
      menuItem("Event Coverage", tabName = "eventcoverage", icon = icon("layer-group")),
      menuItem("Zeta Score", tabName = "zeta", icon = icon("calculator")),
      menuItem("SVM Analysis", tabName = "svm", icon = icon("brain")),
      menuItem("FDR Cutoff", tabName = "fdr", icon = icon("cut")),
      menuItem("Single Cell QC", tabName = "singlecell", icon = icon("mobile")),
      menuItem("Results", tabName = "results", icon = icon("download")),
      menuItem("Help & Support", tabName = "help", icon = icon("question-circle"))
    )
  ),
  
  dashboardBody(
    useShinyjs(),
    tabItems(
      
      # Welcome Tab
      tabItem(tabName = "welcome",
        fluidRow(
          box(title = "Welcome to ZetaSuite", width = 12,
            h3("Multi-dimensional High-throughput Data Analysis"),
            p("ZetaSuite is an R package designed for analyzing multi-dimensional high-throughput screening data, particularly two-dimensional RNAi screens and single-cell RNA sequencing data."),
            br(),
            h4("Key Features:"),
            tags$ul(
              tags$li("Quality Control Analysis"),
              tags$li("Z-score Normalization"),
              tags$li("Event Coverage Analysis"),
              tags$li("Zeta Score Calculation"),
              tags$li("SVM-based Background Correction"),
              tags$li("Screen Strength Analysis"),
              tags$li("Single Cell Quality Control")
            ),
            br(),
            h4("Getting Started:"),
            p("1. Use the 'Example Data' tab to explore the built-in dataset"),
            p("2. Or upload your own data in the 'Data Upload' tab"),
            p("3. Follow the analysis workflow through the tabs"),
            br(),
            p("For more information, see the package vignette and documentation.")
          )
        )
      ),
      
      # Data Upload Tab
      tabItem(tabName = "upload",
        fluidRow(
          box(title = "Upload Data Files", width = 12,
            fileInput("countMat", "Upload Count Matrix (CSV)", accept = ".csv"),
            fileInput("negGene", "Upload Negative Control Genes (CSV)", accept = ".csv"),
            fileInput("posGene", "Upload Positive Control Genes (CSV)", accept = ".csv"),
            fileInput("nonExpGene", "Upload Non-expressed Genes (CSV, optional)", accept = ".csv"),
            actionButton("loadData", "Load Data", class = "btn-primary")
          )
        ),
        fluidRow(
          box(title = "Data Preview", width = 12,
            tabsetPanel(
              tabPanel("Count Matrix", DT::dataTableOutput("countMatPreview")),
              tabPanel("Negative Controls", DT::dataTableOutput("negGenePreview")),
              tabPanel("Positive Controls", DT::dataTableOutput("posGenePreview"))
            )
          )
        )
      ),
      
      # Example Data Tab
      tabItem(tabName = "example",
        fluidRow(
          box(title = "Example Dataset", width = 12,
            h4("HTS2 Screening Dataset"),
            p("This example dataset contains:"),
            tags$ul(
              tags$li("1,609 genes × 100 alternative splicing events"),
              tags$li("30 negative control genes (non-specific siRNAs)"),
              tags$li("20 positive control genes (PTB-targeting siRNAs)"),
              tags$li("50 non-expressed genes (RPKM < 1 in HeLa cells)")
            ),
            br(),
            actionButton("loadExampleData", "Load Example Data", class = "btn-success")
          )
        ),
        fluidRow(
          box(title = "Example Data Summary", width = 12,
            verbatimTextOutput("exampleDataSummary")
          )
        )
      ),
      
      # Quality Control Tab
      tabItem(tabName = "qc",
        fluidRow(
          box(title = "Quality Control Analysis", width = 12,
            actionButton("runQC", "Run Quality Control", class = "btn-success"),
            br(), br(),
            textOutput("qcStatus")
          )
        ),
        fluidRow(
          box(title = "Score Distribution", width = 6,
            plotOutput("scoreQCPlot")
          ),
          box(title = "t-SNE Plot", width = 6,
            plotOutput("tsnePlot")
          )
        ),
        fluidRow(
          box(title = "Box Plots", width = 6,
            plotOutput("boxPlot")
          ),
          box(title = "SSMD Distribution", width = 6,
            plotOutput("ssmdPlot")
          )
        )
      ),
      
      # Z-score Analysis Tab
      tabItem(tabName = "zscore",
        fluidRow(
          box(title = "Z-score Normalization", width = 12,
            actionButton("runZscore", "Calculate Z-scores", class = "btn-success"),
            br(), br(),
            textOutput("zscoreStatus")
          )
        ),
        fluidRow(
          box(title = "Z-score Matrix Preview", width = 12,
            DT::dataTableOutput("zscorePreview")
          )
        )
      ),
      
      # Event Coverage Tab
      tabItem(tabName = "eventcoverage",
        fluidRow(
          box(title = "Event Coverage Parameters", width = 6,
            numericInput("binNum", "Number of Bins", value = 100, min = 10, max = 500),
            checkboxInput("combine", "Combine Directions", value = TRUE),
            actionButton("runEventCoverage", "Calculate Event Coverage", class = "btn-success")
          ),
          box(title = "Event Coverage Status", width = 6,
            textOutput("eventCoverageStatus")
          )
        ),
        fluidRow(
          box(title = "Decrease Direction", width = 6,
            plotOutput("ecDecreasePlot")
          ),
          box(title = "Increase Direction", width = 6,
            plotOutput("ecIncreasePlot")
          )
        )
      ),
      
      # Zeta Score Tab
      tabItem(tabName = "zeta",
        fluidRow(
          box(title = "Zeta Score Parameters", width = 6,
            checkboxInput("useSVM", "Use SVM Curves", value = FALSE),
            actionButton("runZeta", "Calculate Zeta Scores", class = "btn-success")
          ),
          box(title = "Zeta Score Status", width = 6,
            textOutput("zetaStatus")
          )
        ),
        fluidRow(
          box(title = "Zeta Scores Preview", width = 12,
            DT::dataTableOutput("zetaPreview")
          )
        ),
        fluidRow(
          box(title = "Top Hits by Zeta_D", width = 6,
            DT::dataTableOutput("topDecreaseTable")
          ),
          box(title = "Top Hits by Zeta_I", width = 6,
            DT::dataTableOutput("topIncreaseTable")
          )
        )
      ),
      
      # SVM Analysis Tab
      tabItem(tabName = "svm",
        fluidRow(
          box(title = "SVM Analysis", width = 12,
            actionButton("runSVM", "Run SVM Analysis", class = "btn-success"),
            br(), br(),
            textOutput("svmStatus")
          )
        ),
        fluidRow(
          box(title = "SVM Results", width = 12,
            tabsetPanel(
              tabPanel("Decrease Direction", DT::dataTableOutput("svmDecreaseTable")),
              tabPanel("Increase Direction", DT::dataTableOutput("svmIncreaseTable"))
            )
          )
        )
      ),
      
      # FDR Cutoff Tab
      tabItem(tabName = "fdr",
        fluidRow(
          box(title = "FDR Cutoff Parameters", width = 6,
            checkboxInput("combineFDR", "Combine Directions", value = FALSE),
            actionButton("runFDR", "Calculate FDR Cutoffs", class = "btn-success")
          ),
          box(title = "FDR Cutoff Status", width = 6,
            textOutput("fdrStatus")
          )
        ),
        fluidRow(
          box(title = "Zeta Score Distribution by Type", width = 6,
            plotOutput("zetaTypePlot")
          ),
          box(title = "Screen Strength Curves", width = 6,
            plotOutput("ssCutoffPlot")
          )
        ),
        fluidRow(
          box(title = "FDR Cutoff Results", width = 12,
            DT::dataTableOutput("fdrTable")
          )
        ),
        fluidRow(
          box(title = "Hit Selection", width = 12,
            numericInput("ssThreshold", "Screen Strength Threshold", value = 0.8, min = 0, max = 1, step = 0.1),
            actionButton("selectHits", "Select Hits", class = "btn-primary"),
            br(), br(),
            verbatimTextOutput("hitSelectionResults")
          )
        )
      ),
      
      # Single Cell QC Tab
      tabItem(tabName = "singlecell",
        fluidRow(
          box(title = "Single Cell Data Upload", width = 12,
            fileInput("countMatSC", "Upload Single Cell Count Matrix (CSV)", accept = ".csv"),
            actionButton("loadSCData", "Load Single Cell Data", class = "btn-primary")
          )
        ),
        fluidRow(
          box(title = "Single Cell QC Parameters", width = 6,
            numericInput("binNumSC", "Number of Bins", value = 10, min = 5, max = 100),
            checkboxInput("filterSC", "Filter Low Count Cells", value = TRUE),
            actionButton("runSingleCell", "Run Single Cell QC", class = "btn-success")
          ),
          box(title = "Single Cell QC Status", width = 6,
            textOutput("singleCellStatus")
          )
        ),
        fluidRow(
          box(title = "Zeta Score Distribution", width = 12,
            plotOutput("singleCellPlot")
          )
        ),
        fluidRow(
          box(title = "Single Cell Results", width = 12,
            DT::dataTableOutput("singleCellTable")
          )
        )
      ),
      
      # Results Tab
      tabItem(tabName = "results",
        fluidRow(
          box(title = "Download Results", width = 12,
            downloadButton("downloadZscore", "Download Z-scores"),
            downloadButton("downloadZeta", "Download Zeta Scores"),
            downloadButton("downloadFDR", "Download FDR Results"),
            downloadButton("downloadHits", "Download Selected Hits"),
            downloadButton("downloadSingleCell", "Download Single Cell Results"),
            downloadButton("downloadReport", "Download Analysis Report")
          )
        ),
        fluidRow(
          box(title = "Analysis Summary", width = 12,
            verbatimTextOutput("analysisSummary")
          )
        )
      ),
      
      # Help & Support Tab
      tabItem(tabName = "help",
        fluidRow(
          box(title = "Documentation & Support", width = 12,
            h4("Package Documentation"),
            p("For detailed documentation and examples, see the package vignette:"),
            code("vignette(\"ZetaSuite\")"),
            br(),
            p("Official CRAN package page:"),
            tags$a(href = "https://cran.r-project.org/web/packages/ZetaSuite/index.html", 
                   "ZetaSuite on CRAN", 
                   target = "_blank", 
                   class = "btn btn-info"),
            br(), br(),
            h4("Bug Reports & Feature Requests"),
            p("If you encounter any bugs or have feature requests, please report them on our GitHub issues page:"),
            tags$a(href = "https://github.com/JunhuiLi1017/ZetaSuite/issues", 
                   "Report a Bug or Request Feature", 
                   target = "_blank", 
                   class = "btn btn-warning"),
            br(), br(),
            h4("Citation"),
            p("If you use ZetaSuite in your research, please cite:"),
            p("Hao, Y., Zhang, S., Shao, C. et al. ZetaSuite: computational analysis of two-dimensional high-throughput data from multi-target screens and single-cell transcriptomics. Genome Biol 23, 162 (2022). https://doi.org/10.1186/s13059-022-02729-4"),
            br(),
            h4("Contact"),
            p("For questions about the package, contact the maintainer:"),
            p("Junhui Li <ljh.biostat@gmail.com>")
          )
        ),
        fluidRow(
          box(title = "Troubleshooting", width = 12,
            h4("Common Issues"),
            tags$ul(
              tags$li("Make sure all required packages are installed"),
              tags$li("Check that CSV files have correct column headers and data types"),
              tags$li("For large datasets, consider reducing the number of bins"),
              tags$li("Ensure you have sufficient positive and negative control samples")
            ),
            br(),
            h4("Data Format Requirements"),
            p("Count Matrix: Rows = Genes/siRNAs, Columns = Readouts/conditions"),
            p("Control Files: First column should contain gene/siRNA identifiers"),
            p("All files should be in CSV format with proper headers")
          )
        )
      )
    )
  )
)

# Server Logic
server <- function(input, output, session) {
  
  # Reactive values to store data and results
  values <- reactiveValues(
    countMat = NULL,
    negGene = NULL,
    posGene = NULL,
    nonExpGene = NULL,
    countMatSC = NULL,
    zscoreVal = NULL,
    ecData = NULL,
    zetaData = NULL,
    svmData = NULL,
    fdrData = NULL,
    singleCellData = NULL,
    qcResults = NULL,
    selectedHits = NULL
  )
  
  # Load example data
  observeEvent(input$loadExampleData, {
    tryCatch({
      # Load ZetaSuite package data
      library(ZetaSuite)
      data(countMat)
      data(negGene)
      data(posGene)
      data(nonExpGene)
      data(ZseqList)
      data(SVMcurve)
      
      values$countMat <- countMat
      values$negGene <- negGene
      values$posGene <- posGene
      values$nonExpGene <- nonExpGene
      
      showNotification("Example data loaded successfully!", type = "message")
    }, error = function(e) {
      showNotification(paste("Error loading example data:", e$message), type = "error")
    })
  })
  
  # Load main data
  observeEvent(input$loadData, {
    req(input$countMat, input$negGene, input$posGene)
    
    tryCatch({
      values$countMat <- read.csv(input$countMat$datapath, row.names = 1)
      values$negGene <- read.csv(input$negGene$datapath)
      values$posGene <- read.csv(input$posGene$datapath)
      
      if (!is.null(input$nonExpGene)) {
        values$nonExpGene <- read.csv(input$nonExpGene$datapath)
      }
      
      showNotification("Data loaded successfully!", type = "message")
    }, error = function(e) {
      showNotification(paste("Error loading data:", e$message), type = "error")
    })
  })
  
  # Load single cell data
  observeEvent(input$loadSCData, {
    req(input$countMatSC)
    
    tryCatch({
      values$countMatSC <- read.csv(input$countMatSC$datapath, row.names = 1)
      showNotification("Single cell data loaded successfully!", type = "message")
    }, error = function(e) {
      showNotification(paste("Error loading single cell data:", e$message), type = "error")
    })
  })
  
  # Example data summary
  output$exampleDataSummary <- renderPrint({
    if (!is.null(values$countMat)) {
      cat("=== Example Dataset Summary ===\n\n")
      cat("Count matrix dimensions:", dim(values$countMat), "\n")
      cat("Negative controls:", nrow(values$negGene), "genes\n")
      cat("Positive controls:", nrow(values$posGene), "genes\n")
      if (!is.null(values$nonExpGene)) {
        cat("Non-expressed genes:", nrow(values$nonExpGene), "genes\n")
      }
      cat("\nData ready for analysis!\n")
    } else {
      cat("Click 'Load Example Data' to load the built-in dataset.\n")
    }
  })
  
  # Data preview outputs
  output$countMatPreview <- DT::renderDataTable({
    req(values$countMat)
    DT::datatable(head(values$countMat, 10), options = list(scrollX = TRUE))
  })
  
  output$negGenePreview <- DT::renderDataTable({
    req(values$negGene)
    DT::datatable(values$negGene, options = list(scrollX = TRUE))
  })
  
  output$posGenePreview <- DT::renderDataTable({
    req(values$posGene)
    DT::datatable(values$posGene, options = list(scrollX = TRUE))
  })
  
  # Quality Control
  observeEvent(input$runQC, {
    req(values$countMat, values$negGene, values$posGene)
    
    tryCatch({
      withProgress(message = "Running Quality Control...", {
        values$qcResults <- QC(values$countMat, values$negGene, values$posGene)
      })
      output$qcStatus <- renderText("Quality Control completed successfully!")
      showNotification("Quality Control completed!", type = "message")
    }, error = function(e) {
      output$qcStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in Quality Control:", e$message), type = "error")
    })
  })
  
  # Z-score Analysis
  observeEvent(input$runZscore, {
    req(values$countMat, values$negGene)
    
    tryCatch({
      withProgress(message = "Calculating Z-scores...", {
        values$zscoreVal <- Zscore(values$countMat, values$negGene)
      })
      output$zscoreStatus <- renderText("Z-score calculation completed successfully!")
      showNotification("Z-scores calculated!", type = "message")
    }, error = function(e) {
      output$zscoreStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in Z-score calculation:", e$message), type = "error")
    })
  })
  
  # Event Coverage
  observeEvent(input$runEventCoverage, {
    req(values$zscoreVal, values$negGene, values$posGene)
    
    tryCatch({
      withProgress(message = "Calculating Event Coverage...", {
        values$ecData <- EventCoverage(values$zscoreVal, values$negGene, values$posGene, 
                                      input$binNum, input$combine)
      })
      output$eventCoverageStatus <- renderText("Event Coverage calculation completed successfully!")
      showNotification("Event Coverage calculated!", type = "message")
    }, error = function(e) {
      output$eventCoverageStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in Event Coverage calculation:", e$message), type = "error")
    })
  })
  
  # Zeta Score
  observeEvent(input$runZeta, {
    req(values$zscoreVal, values$ecData)
    
    tryCatch({
      withProgress(message = "Calculating Zeta Scores...", {
        if (input$useSVM && !is.null(values$svmData)) {
          values$zetaData <- Zeta(values$zscoreVal, values$ecData[[1]]$ZseqList, 
                                 values$svmData, SVM = TRUE)
        } else {
          values$zetaData <- Zeta(values$zscoreVal, values$ecData[[1]]$ZseqList, SVM = FALSE)
        }
      })
      output$zetaStatus <- renderText("Zeta Score calculation completed successfully!")
      showNotification("Zeta Scores calculated!", type = "message")
    }, error = function(e) {
      output$zetaStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in Zeta Score calculation:", e$message), type = "error")
    })
  })
  
  # SVM Analysis
  observeEvent(input$runSVM, {
    req(values$ecData)
    
    tryCatch({
      withProgress(message = "Running SVM Analysis...", {
        values$svmData <- SVM(values$ecData)
      })
      output$svmStatus <- renderText("SVM Analysis completed successfully!")
      showNotification("SVM Analysis completed!", type = "message")
    }, error = function(e) {
      output$svmStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in SVM Analysis:", e$message), type = "error")
    })
  })
  
  # FDR Cutoff
  observeEvent(input$runFDR, {
    req(values$zetaData, values$negGene, values$posGene, values$nonExpGene)
    
    tryCatch({
      withProgress(message = "Calculating FDR Cutoffs...", {
        values$fdrData <- FDRcutoff(values$zetaData, values$negGene, values$posGene, 
                                   values$nonExpGene, input$combineFDR)
      })
      output$fdrStatus <- renderText("FDR Cutoff calculation completed successfully!")
      showNotification("FDR Cutoffs calculated!", type = "message")
    }, error = function(e) {
      output$fdrStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in FDR Cutoff calculation:", e$message), type = "error")
    })
  })
  
  # Hit Selection
  observeEvent(input$selectHits, {
    req(values$zetaData, values$fdrData)
    
    tryCatch({
      fdr_table <- values$fdrData[[1]]
      selected_threshold <- fdr_table[fdr_table$SS >= input$ssThreshold, ]
      
      if (nrow(selected_threshold) > 0) {
        best_threshold <- selected_threshold[which.max(selected_threshold$SS), ]
        combined_zeta <- values$zetaData$Zeta_D + values$zetaData$Zeta_I
        hits <- names(combined_zeta[combined_zeta >= best_threshold$Cut_Off])
        values$selectedHits <- data.frame(
          Gene = hits,
          Zeta_D = values$zetaData[hits, "Zeta_D"],
          Zeta_I = values$zetaData[hits, "Zeta_I"],
          Combined_Zeta = combined_zeta[hits],
          stringsAsFactors = FALSE
        )
        
        output$hitSelectionResults <- renderPrint({
          cat("=== Hit Selection Results ===\n\n")
          cat("Screen Strength threshold:", input$ssThreshold, "\n")
          cat("Selected threshold:", best_threshold$Cut_Off, "\n")
          cat("Screen Strength:", best_threshold$SS, "\n")
          cat("Total hits identified:", length(hits), "\n")
          cat("Apparent FDR:", best_threshold$aFDR, "\n")
        })
        
        showNotification(paste("Selected", length(hits), "hits!"), type = "message")
      } else {
        output$hitSelectionResults <- renderPrint({
          cat("No thresholds found with Screen Strength >=", input$ssThreshold, "\n")
          cat("Try lowering the threshold or running FDR analysis with different parameters.\n")
        })
      }
    }, error = function(e) {
      output$hitSelectionResults <- renderPrint({
        cat("Error in hit selection:", e$message, "\n")
      })
    })
  })
  
  # Single Cell QC
  observeEvent(input$runSingleCell, {
    req(values$countMatSC)
    
    tryCatch({
      withProgress(message = "Running Single Cell QC...", {
        values$singleCellData <- ZetaSuitSC(values$countMatSC, input$binNumSC, input$filterSC)
      })
      output$singleCellStatus <- renderText("Single Cell QC completed successfully!")
      showNotification("Single Cell QC completed!", type = "message")
    }, error = function(e) {
      output$singleCellStatus <- renderText(paste("Error:", e$message))
      showNotification(paste("Error in Single Cell QC:", e$message), type = "error")
    })
  })
  
  # Plot outputs
  output$scoreQCPlot <- renderPlot({
    req(values$qcResults)
    tryCatch({
      if (is.null(values$qcResults$score_qc)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No score QC plot available", cex = 1.2)
      } else {
        values$qcResults$score_qc
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$tsnePlot <- renderPlot({
    req(values$qcResults)
    tryCatch({
      if (is.null(values$qcResults$tSNE_QC)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No t-SNE plot available", cex = 1.2)
      } else {
        values$qcResults$tSNE_QC
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$boxPlot <- renderPlot({
    req(values$qcResults)
    tryCatch({
      if (is.null(values$qcResults$QC_box)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No box plot available", cex = 1.2)
      } else {
        values$qcResults$QC_box
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$ssmdPlot <- renderPlot({
    req(values$qcResults)
    tryCatch({
      if (is.null(values$qcResults$QC_SSMD)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No SSMD plot available", cex = 1.2)
      } else {
        values$qcResults$QC_SSMD
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$ecDecreasePlot <- renderPlot({
    req(values$ecData)
    tryCatch({
      if (is.null(values$ecData[[2]]$EC_jitter_D)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No decrease direction plot available", cex = 1.2)
      } else {
        values$ecData[[2]]$EC_jitter_D
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$ecIncreasePlot <- renderPlot({
    req(values$ecData)
    tryCatch({
      if (is.null(values$ecData[[2]]$EC_jitter_I)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No increase direction plot available", cex = 1.2)
      } else {
        values$ecData[[2]]$EC_jitter_I
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$zetaTypePlot <- renderPlot({
    req(values$fdrData)
    tryCatch({
      if (is.null(values$fdrData[[2]]$Zeta_type)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No zeta type plot available", cex = 1.2)
      } else {
        values$fdrData[[2]]$Zeta_type
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$ssCutoffPlot <- renderPlot({
    req(values$fdrData)
    tryCatch({
      if (is.null(values$fdrData[[2]]$SS_cutOff)) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No screen strength plot available", cex = 1.2)
      } else {
        values$fdrData[[2]]$SS_cutOff
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  output$singleCellPlot <- renderPlot({
    req(values$singleCellData)
    tryCatch({
      if (is.null(values$singleCellData[[2]])) {
        plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
        text(1, 1, "No single cell plot available", cex = 1.2)
      } else {
        values$singleCellData[[2]]
      }
    }, error = function(e) {
      plot(1, 1, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(1, 1, paste("Error rendering plot:", e$message), cex = 1.2)
    })
  })
  
  # Table outputs
  output$zscorePreview <- DT::renderDataTable({
    req(values$zscoreVal)
    DT::datatable(head(values$zscoreVal, 10), options = list(scrollX = TRUE))
  })
  
  output$zetaPreview <- DT::renderDataTable({
    req(values$zetaData)
    DT::datatable(head(values$zetaData, 10), options = list(scrollX = TRUE))
  })
  
  output$topDecreaseTable <- DT::renderDataTable({
    req(values$zetaData)
    top_decrease <- head(values$zetaData[order(values$zetaData$Zeta_D, decreasing = TRUE), ], 10)
    DT::datatable(top_decrease, options = list(scrollX = TRUE))
  })
  
  output$topIncreaseTable <- DT::renderDataTable({
    req(values$zetaData)
    top_increase <- head(values$zetaData[order(values$zetaData$Zeta_I, decreasing = TRUE), ], 10)
    DT::datatable(top_increase, options = list(scrollX = TRUE))
  })
  
  output$svmDecreaseTable <- DT::renderDataTable({
    req(values$svmData)
    DT::datatable(values$svmData$cutOffD, options = list(scrollX = TRUE))
  })
  
  output$svmIncreaseTable <- DT::renderDataTable({
    req(values$svmData)
    DT::datatable(values$svmData$cutOffI, options = list(scrollX = TRUE))
  })
  
  output$fdrTable <- DT::renderDataTable({
    req(values$fdrData)
    DT::datatable(values$fdrData[[1]], options = list(scrollX = TRUE))
  })
  
  output$singleCellTable <- DT::renderDataTable({
    req(values$singleCellData)
    DT::datatable(values$singleCellData[[1]], options = list(scrollX = TRUE))
  })
  
  # Download handlers
  output$downloadZscore <- downloadHandler(
    filename = function() { "zscore_results.csv" },
    content = function(file) {
      req(values$zscoreVal)
      write.csv(values$zscoreVal, file)
    }
  )
  
  output$downloadZeta <- downloadHandler(
    filename = function() { "zeta_scores.csv" },
    content = function(file) {
      req(values$zetaData)
      write.csv(values$zetaData, file)
    }
  )
  
  output$downloadFDR <- downloadHandler(
    filename = function() { "fdr_results.csv" },
    content = function(file) {
      req(values$fdrData)
      write.csv(values$fdrData[[1]], file)
    }
  )
  
  output$downloadHits <- downloadHandler(
    filename = function() { "selected_hits.csv" },
    content = function(file) {
      req(values$selectedHits)
      write.csv(values$selectedHits, file, row.names = FALSE)
    }
  )
  
  output$downloadSingleCell <- downloadHandler(
    filename = function() { "single_cell_results.csv" },
    content = function(file) {
      req(values$singleCellData)
      write.csv(values$singleCellData[[1]], file)
    }
  )
  
  output$downloadReport <- downloadHandler(
    filename = function() { "zetaSuite_analysis_report.txt" },
    content = function(file) {
      cat("=== ZetaSuite Analysis Report ===\n\n", file = file)
      
      if (!is.null(values$countMat)) {
        cat("Data Summary:\n", file = file, append = TRUE)
        cat("Count matrix dimensions:", dim(values$countMat), "\n", file = file, append = TRUE)
        cat("Negative controls:", nrow(values$negGene), "genes\n", file = file, append = TRUE)
        cat("Positive controls:", nrow(values$posGene), "genes\n", file = file, append = TRUE)
        if (!is.null(values$nonExpGene)) {
          cat("Non-expressed genes:", nrow(values$nonExpGene), "genes\n", file = file, append = TRUE)
        }
        cat("\n", file = file, append = TRUE)
      }
      
      if (!is.null(values$zetaData)) {
        cat("Zeta Score Summary:\n", file = file, append = TRUE)
        cat("Number of genes:", nrow(values$zetaData), "\n", file = file, append = TRUE)
        cat("Zeta_D range:", range(values$zetaData$Zeta_D), "\n", file = file, append = TRUE)
        cat("Zeta_I range:", range(values$zetaData$Zeta_I), "\n", file = file, append = TRUE)
        cat("\n", file = file, append = TRUE)
      }
      
      if (!is.null(values$selectedHits)) {
        cat("Hit Selection Results:\n", file = file, append = TRUE)
        cat("Total hits identified:", nrow(values$selectedHits), "\n", file = file, append = TRUE)
        cat("\n", file = file, append = TRUE)
      }
      
      cat("Analysis completed on:", Sys.time(), "\n", file = file, append = TRUE)
    }
  )
  
  # Analysis summary
  output$analysisSummary <- renderPrint({
    cat("=== ZetaSuite Analysis Summary ===\n\n")
    
    if (!is.null(values$countMat)) {
      cat("✓ Count matrix loaded:", nrow(values$countMat), "genes ×", ncol(values$countMat), "samples\n")
    }
    
    if (!is.null(values$zscoreVal)) {
      cat("✓ Z-scores calculated\n")
    }
    
    if (!is.null(values$ecData)) {
      cat("✓ Event coverage calculated\n")
    }
    
    if (!is.null(values$zetaData)) {
      cat("✓ Zeta scores calculated\n")
    }
    
    if (!is.null(values$svmData)) {
      cat("✓ SVM analysis completed\n")
    }
    
    if (!is.null(values$fdrData)) {
      cat("✓ FDR cutoffs calculated\n")
    }
    
    if (!is.null(values$selectedHits)) {
      cat("✓ Hits selected:", nrow(values$selectedHits), "genes\n")
    }
    
    if (!is.null(values$singleCellData)) {
      cat("✓ Single cell QC completed\n")
    }
    
    cat("\nAnalysis ready for download!")
  })
}

# Run the application
shinyApp(ui = ui, server = server) 

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(warning = FALSE, message = FALSE, fig.width = 8, fig.height = 6) 

## ----install, eval=FALSE------------------------------------------------------
# # Install from CRAN
# install.packages("ZetaSuite")
# 
# # Load the package
# library(ZetaSuite)

## ----shiny_app, eval=FALSE----------------------------------------------------
# # Launch the Shiny app
# ZetaSuiteApp()
# 
# # Launch without opening browser automatically
# ZetaSuiteApp(launch.browser = FALSE)
# 
# # Launch on a specific port
# ZetaSuiteApp(port = 3838)

## ----load_data----------------------------------------------------------------
library(ZetaSuite)

# Load example data
data(countMat)
data(negGene)
data(posGene)
data(nonExpGene)
data(ZseqList)
data(SVMcurve)

# Display data dimensions
cat("Count matrix dimensions:", dim(countMat), "\n")
cat("Negative controls:", nrow(negGene), "genes\n")
cat("Positive controls:", nrow(posGene), "genes\n")
cat("Non-expressed genes:", nrow(nonExpGene), "genes\n")

## ----qc_analysis--------------------------------------------------------------
# Perform quality control analysis
qc_results <- QC(countMat, negGene, posGene)

# Display QC plots
cat("QC analysis completed. Generated", length(qc_results), "diagnostic plots.\n")

## ----qc_plot1, fig.height = 5, fig.width = 8----------------------------------
qc_results$score_qc

## ----qc_plot2, fig.height = 5, fig.width = 6----------------------------------
qc_results$tSNE_QC

## ----qc_plot3, fig.height = 4, fig.width = 8----------------------------------
grid::grid.draw(qc_results$QC_box)

## ----qc_plot4, fig.height = 5, fig.width = 6----------------------------------
qc_results$QC_SSMD

## ----zscore_analysis----------------------------------------------------------
# Calculate Z-scores
zscore_matrix <- Zscore(countMat, negGene)

# Display first few rows and columns
cat("Z-score matrix dimensions:", dim(zscore_matrix), "\n")
cat("First 5 rows and columns:\n")
print(zscore_matrix[1:5, 1:5])

## ----event_coverage-----------------------------------------------------------
# Calculate event coverage
ec_results <- EventCoverage(zscore_matrix, negGene, posGene, binNum = 100, combine = TRUE)

# Display event coverage plots
cat("Event coverage analysis completed.\n")

## ----ec_plots, fig.height = 5, fig.width = 10---------------------------------
# Decrease direction (exon skipping)
ec_results[[2]]$EC_jitter_D

## ----ec_plots2, fig.height = 5, fig.width = 10--------------------------------
# Increase direction (exon inclusion)
ec_results[[2]]$EC_jitter_I

## ----zeta_calculation---------------------------------------------------------
# Calculate zeta scores without SVM correction
zeta_scores <- Zeta(zscore_matrix, ZseqList, SVM = FALSE)

# Display summary statistics
cat("Zeta score summary:\n")
cat("Number of genes:", nrow(zeta_scores), "\n")
cat("Zeta_D range:", range(zeta_scores$Zeta_D), "\n")
cat("Zeta_I range:", range(zeta_scores$Zeta_I), "\n")

# Show top hits
cat("\nTop 10 genes by Zeta_D (decrease direction):\n")
top_decrease <- head(zeta_scores[order(zeta_scores$Zeta_D, decreasing = TRUE), ], 10)
print(top_decrease)

## ----svm_analysis, eval=FALSE-------------------------------------------------
# # Run SVM analysis (can be computationally intensive)
# svm_results <- SVM(ec_results)
# 
# # Calculate zeta scores with SVM correction
# zeta_scores_svm <- Zeta(zscore_matrix, ZseqList, SVMcurve = svm_results, SVM = TRUE)

## ----screen_strength----------------------------------------------------------
# Calculate FDR cutoffs and Screen Strength
fdr_results <- FDRcutoff(zeta_scores, negGene, posGene, nonExpGene, combine = TRUE)

# Display Screen Strength plots
cat("Screen Strength analysis completed.\n")

## ----ss_plot1, fig.height = 5, fig.width = 8----------------------------------
fdr_results[[2]]$Zeta_type

## ----ss_plot2, fig.height = 5, fig.width = 6----------------------------------
fdr_results[[2]]$SS_cutOff

## ----fdr_table----------------------------------------------------------------
# Display FDR cutoff results
fdr_table <- fdr_results[[1]]
cat("FDR cutoff results summary:\n")
cat("Number of thresholds tested:", nrow(fdr_table), "\n")
cat("Screen Strength range:", range(fdr_table$SS), "\n")

# Show optimal thresholds (SS > 0.8)
optimal_thresholds <- fdr_table[fdr_table$SS > 0.8, ]
cat("\nOptimal thresholds (SS > 0.8):\n")
print(head(optimal_thresholds, 10))

## ----hit_selection------------------------------------------------------------
# Example: Select threshold with SS > 0.8 and reasonable number of hits
selected_threshold <- fdr_table[fdr_table$SS > 0.8 & fdr_table$TotalHits > 50, ]
if(nrow(selected_threshold) > 0) {
  best_threshold <- selected_threshold[which.max(selected_threshold$SS), ]
  cat("Recommended threshold:", best_threshold$Cut_Off, "\n")
  cat("Screen Strength:", best_threshold$SS, "\n")
  cat("Total hits:", best_threshold$TotalHits, "\n")
  
  # Identify hits
  combined_zeta <- zeta_scores$Zeta_D + zeta_scores$Zeta_I
  hits <- names(combined_zeta[combined_zeta >= best_threshold$Cut_Off])
  cat("Number of hits identified:", length(hits), "\n")
}

## ----single_cell_example, eval=FALSE------------------------------------------
# # Example single cell analysis (requires single cell count matrix)
# # single_cell_results <- ZetaSuitSC(count_matrix_sc, binNum = 10, filter = TRUE)

## ----custom_params, eval=FALSE------------------------------------------------
# # Custom event coverage analysis
# ec_custom <- EventCoverage(zscore_matrix, negGene, posGene,
#                           binNum = 200,    # More bins for finer resolution
#                           combine = FALSE) # Separate decrease/increase directions
# 
# # Custom zeta score calculation with SVM
# zeta_custom <- Zeta(zscore_matrix, ZseqList,
#                    SVMcurve = svm_results,
#                    SVM = TRUE)  # Use SVM correction
# 
# # Custom FDR analysis
# fdr_custom <- FDRcutoff(zeta_scores, negGene, posGene, nonExpGene,
#                        combine = FALSE)  # Analyze directions separately

## ----batch_processing, eval=FALSE---------------------------------------------
# # Example: Process multiple datasets
# datasets <- list(dataset1 = list(countMat = countMat1, negGene = negGene1),
#                 dataset2 = list(countMat = countMat2, negGene = negGene2))
# 
# results <- lapply(datasets, function(ds) {
#   zscore <- Zscore(ds$countMat, ds$negGene)
#   ec <- EventCoverage(zscore, ds$negGene, ds$posGene, binNum = 100)
#   zeta <- Zeta(zscore, ZseqList, SVM = FALSE)
#   return(list(zscore = zscore, ec = ec, zeta = zeta))
# })

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' Perform quality control analysis for high-throughput screening data.
#'
#' This function performs comprehensive quality control analysis on high-throughput screening data to evaluate experimental design and data quality. It generates multiple diagnostic plots and calculates SSMD (Strictly Standardized Mean Difference) scores to assess the separation between positive and negative controls.
#'
#' @param countMat A matrix of raw count data where rows represent genes/siRNAs and columns represent readouts/conditions. The matrix should have row names corresponding to gene/siRNA identifiers.
#'
#' @param negGene A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in countMat.
#'
#' @param posGene A data frame or matrix containing positive control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in countMat.
#'
#' @return A list containing four diagnostic plots:
#'   \item{score_qc}{A jitter plot showing the distribution of raw scores across all readouts for positive and negative controls}
#'   \item{tSNE_QC}{A t-SNE plot showing the global separation of positive and negative control samples in 2D space}
#'   \item{QC_box}{Side-by-side boxplots showing the distribution of scores for positive and negative controls across all readouts}
#'   \item{QC_SSMD}{A density plot showing the distribution of SSMD scores across readouts, with a threshold line at SSMD=2 and the percentage of high-quality readouts displayed}
#'
#' @details The function performs the following quality control analyses:
#' \enumerate{
#'   \item Creates jitter plots to visualize score distributions across readouts
#'   \item Performs t-SNE dimensionality reduction to assess global sample separation
#'   \item Generates boxplots to compare score distributions between control groups
#'   \item Calculates SSMD scores for each readout: \eqn{\mathrm{SSMD} = (\mu_{pos} - \mu_{neg}) / \sqrt{\sigma_{pos}^2 + \sigma_{neg}^2}}
#'   \item Reports the percentage of readouts with \eqn{|\mathrm{SSMD}| \ge 2} (considered high quality)
#' }
#' 
#' SSMD scores \eqn{\ge 2} indicate good separation between positive and negative controls, suggesting high-quality readouts.
#'
#' @references
#' Laurens van der Maaten & Geoffrey Hinton: Visualizing Data using t-SNE. Journal of Machine Learning Research 2008, 9(2008):2579-2605.
#'
#' Zhang XD: A pair of new statistical parameters for quality control in RNA interference high-throughput screening assays. Genomics 2007, 89:552-561.
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(countMat)
#' data(negGene)
#' data(posGene)
#' \donttest{QC(countMat, negGene, posGene)}
#'
#' @keywords ZetaSuite quality control SSMD t-SNE
#'
#' @importFrom ggplot2 ggplot geom_jitter scale_color_manual theme_bw theme xlab ylab geom_point geom_boxplot geom_hline ggtitle geom_density geom_vline annotate ylim aes_string element_blank
#' @importFrom reshape2 melt
#' @importFrom Rtsne Rtsne
#'
#' @importFrom grDevices colorRampPalette dev.off pdf png
#'
#' @importFrom stats quantile var
#'
#' @name QC
#' @export QC

QC <- function (countMat, negGene, posGene){
    countMatNeg <- countMat[rownames(countMat) %in% negGene[,1], ]
    countMatNeg$Type <- rep("Negative", nrow(countMatNeg))
    countMatPos <- countMat[rownames(countMat) %in% posGene[,1], ]
    countMatPos$Type <- rep("Positive", nrow(countMatPos))
    countMatCom <- rbind(countMatNeg, countMatPos)
    countMatCom <- countMatCom[, -1]
    countMatCom[is.na(countMatCom)] <- 0
    countMatComNoNA <- countMatCom
    meltdata <- melt(countMatComNoNA, id = "Type")
    meltdata$value <- as.numeric(as.character(meltdata$value))
    p_score_qc <- ggplot(meltdata) + geom_jitter(aes_string(x = "variable", y = "value", col = "Type"), size = 0.1) + scale_color_manual(values = c("#5aae61", "#c2a5cf")) + theme_bw() + theme(axis.text.x = element_blank(), panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + xlab("") + ylab("")
    Labels <- countMatComNoNA$Type
    #set.seed(42)
    tsne <- Rtsne(countMatComNoNA[, seq(1, ncol(countMatComNoNA) - 1)], dims = 2, perplexity = 30, verbose = TRUE, max_iter = 10000)
    tSNEdata <- as.data.frame(cbind(tsne$Y, countMatComNoNA$Type),stringsAsFactors = FALSE)
    tSNEdata$V3 <- as.factor(tSNEdata$V3)
    tSNEdata$V1 <- as.numeric(tSNEdata$V1)
    tSNEdata$V2 <- as.numeric(tSNEdata$V2)
    p_tSNE_QC <- ggplot(tSNEdata) + geom_point(aes_string(x = "V1", y = "V2", col = "V3"), size = 1) + theme_bw() + scale_color_manual(labels = c("Negative", "Positive"), values = c("#5aae61", "#c2a5cf")) + theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(), legend.title = element_blank()) + xlab("tSNE-1") + ylab("tSNE-2")
    Negative <- countMatComNoNA[countMatComNoNA$Type == "Negative",]
    meltNegative <- melt(Negative, id = "Type")
    Positive <- countMatComNoNA[countMatComNoNA$Type == "Positive",]
    meltPositive <- melt(Positive, id = "Type")
    p1 <- ggplot2::ggplot(meltNegative, aes_string(x = "variable", y = "value")) + geom_boxplot(outlier.shape = NA, col = "#5aae61") + geom_hline(yintercept = 0, col = "red3", linetype = "dashed") + theme_bw() + theme(axis.text.x = element_blank(), panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + xlab("Readouts") + ylab("Input score") + ggtitle("Negative")
    p2 <- ggplot2::ggplot(meltPositive, aes_string(x = "variable", y = "value")) + geom_boxplot(outlier.shape = NA, col = "#c2a5cf") + geom_hline(yintercept = 0, col = "red3", linetype = "dashed") + theme_bw() + theme(axis.text.x = element_blank(), panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + xlab("Readouts") + ylab("Input score") + ggtitle("Positive")
    p_QC_box <- gridExtra::grid.arrange(p1, p2, nrow = 1)
    SSMD <- matrix(0, (length(Negative[1, ]) - 1), 1)
    for (i in seq(1, (length(Negative[1, ]) - 1))) {
        SSMD[i, 1] = (mean(Positive[, i]) - mean(Negative[, i]))/sqrt(var(Positive[, i]) + var(Negative[, i]))
    }
    SSMD <- as.data.frame(SSMD,stringsAsFactors = FALSE)
    row.names(SSMD) <- colnames(countMatComNoNA)[1:(length(Negative[1, ]) - 1)]
    percentage <- length(SSMD[abs(SSMD[, 1]) >= 2, ])/length(SSMD[, 1])
    labels = paste("SSMD>=2(%) is", percentage, "")
    p_QC_SSMD <- ggplot2::ggplot(SSMD) + geom_density(aes_string(x = abs(SSMD[, 1])), fill = "#43a2ca", alpha = 0.5) + geom_vline(xintercept = 2, col = "red", linetype = "dashed") + theme_bw() + theme(axis.text.x = element_blank(), panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + xlab("SSMD values") + ylab("Density") + annotate("text", x = quantile(SSMD[, 1], probs = 0.75), y = 0.75, label = labels) + ylim(0, 1)
    resultList <- list()
    resultList["score_qc"] <- list(p_score_qc)
    resultList["tSNE_QC"] <- list(p_tSNE_QC)
    resultList["QC_box"] <- list(p_QC_box)
    resultList["QC_SSMD"] <- list(p_QC_SSMD)
    return(resultList)
}

#' Generate SVM decision boundaries for positive and negative control separation.
#'
#' This function constructs Support Vector Machine (SVM) models to find optimal decision boundaries that separate positive and negative control samples in event coverage space. It uses radial kernel SVM to create non-linear decision boundaries for both decrease and increase directions.
#'
#' @param ECdataList A list containing event coverage data from the EventCoverage() function. The list should contain:
#'   \itemize{
#'     \item EC_N_D: Event coverage matrix for negative controls in decrease direction
#'     \item EC_P_D: Event coverage matrix for positive controls in decrease direction
#'     \item EC_N_I: Event coverage matrix for negative controls in increase direction
#'     \item EC_P_I: Event coverage matrix for positive controls in increase direction
#'     \item ZseqList: Z-score thresholds for both directions
#'   }
#'
#' @return A list containing two data frames:
#'   \item{cutOffD}{A data frame with SVM decision boundary points for decrease direction. Each row contains a Z-score threshold and the corresponding event coverage threshold that separates positive and negative controls.}
#'   \item{cutOffI}{A data frame with SVM decision boundary points for increase direction. Each row contains a Z-score threshold and the corresponding event coverage threshold that separates positive and negative controls.}
#'
#' @details The function performs the following steps:
#' \enumerate{
#'   \item Prepares training data by combining positive and negative control event coverage data
#'   \item Trains separate SVM models for decrease and increase directions using radial kernel
#'   \item Uses pre-tuned hyperparameters (cost=20, gamma=3 for decrease; cost=50, gamma=2 for increase)
#'   \item Generates prediction grids across the Z-score and event coverage space
#'   \item Identifies decision boundary points where the SVM prediction changes from negative to positive
#'   \item Returns the optimal threshold points for each Z-score bin
#' }
#' 
#' The resulting SVM curves can be used for background correction in zeta score calculations to improve the accuracy of hit identification.
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(countMat)
#' data(negGene)
#' data(posGene)
#' ZscoreVal <- Zscore(countMat, negGene)
#' ECdataList <- EventCoverage(ZscoreVal, negGene, posGene, binNum=10, combine=TRUE)
#' \donttest{SVM(ECdataList)}
#'
#' @keywords ZetaSuite SVM support vector machine decision boundary
#'
#' @importFrom e1071 svm
#' @importFrom reshape2 melt
#'
#' @importFrom stats predict
#'
#' @name SVM
#' @export SVM

SVM <- function(ECdataList){
  old <- options()
  on.exit(options(old))
  options(digits=15)
  EC_N_D <- melt(ECdataList[[1]][["EC_N_D"]])
  EC_P_D <- melt(ECdataList[[1]][["EC_P_D"]])
  P_D <- cbind(EC_P_D,rep("Positive",nrow(EC_P_D)))
  N_D <- cbind(EC_N_D,rep("Negative",nrow(EC_N_D)))
  P_D <- P_D[,-1]
  N_D <- N_D[,-1]
  colnames(P_D) <- c("Zscore","Percent","label")
  colnames(N_D) <- c("Zscore","Percent","label")
  SVMInput_D <- rbind(P_D,N_D)
  SVMInput_D$label <- as.factor(SVMInput_D$label)

  EC_N_I <- melt(ECdataList[[1]][["EC_N_I"]])
  EC_P_I <- melt(ECdataList[[1]][["EC_P_I"]])
  P_I <- cbind(EC_P_I,rep("Positive",nrow(EC_P_I)))
  N_I <- cbind(EC_N_I,rep("Negative",nrow(EC_N_I)))
  P_I <- P_I[,-1]
  N_I <- N_I[,-1]
  colnames(P_I)<-c("Zscore","Percent","label")
  colnames(N_I)<-c("Zscore","Percent","label")
  SVMInput_I <- rbind(P_I,N_I)
  SVMInput_I$label <- as.factor(SVMInput_I$label)

  svm_D_tune <- svm(label ~ ., data=SVMInput_D, type="C-classification", kernel="radial", cost=20, gamma=3, scale=FALSE,probability=TRUE)
  #svm_model_after_tune
  grid <- expand.grid(seq(min(SVMInput_D[,1]), max(SVMInput_D[,1]),length.out=length(unique(SVMInput_D[,1]))), seq(min(SVMInput_D[,2]), max(SVMInput_D[,2]),length.out=length(unique(SVMInput_D[,1]))))
  names(grid) <- names(SVMInput_D)[1:2]
  preds <- predict(svm_D_tune, grid)
  svm_line_D <- data.frame(grid, preds,stringsAsFactors = FALSE)

  svm_I_tune <- svm(label ~ ., data=SVMInput_I, type="C-classification", kernel="radial", cost=50, gmma=2, scale=FALSE,probability=TRUE)
  grid <- expand.grid(seq(min(SVMInput_I[,1]), max(SVMInput_I[,1]),length.out=length(unique(SVMInput_I[,1]))), seq(min(SVMInput_I[,2]), max(SVMInput_I[,2]),length.out=length(unique(SVMInput_I[,1]))))
  names(grid) <- names(SVMInput_I)[1:2]
  preds <- predict(svm_I_tune, grid)
  svm_line_I <- data.frame(grid, preds,stringsAsFactors = FALSE)

  ZseqList <- ECdataList[[1]][["ZseqList"]]
  lineCutOffD <- NULL
  lineCutOffI <- NULL
  for(j in 1:nrow(ZseqList)){
    numberD <- svm_line_D[,1]-ZseqList[j,1]
    numberI <- svm_line_I[,1]-ZseqList[j,2]
    indexD <- which(numberD <= 0.00000001 & numberD >= -0.00000001)
    indexI <- which(numberI <= 0.00000001 & numberI >= -0.00000001)
    if(length(indexD)>0){
      indexD_neg <- indexD[svm_line_D[indexD,3]=="Negative"]
      indexD_neg_max <- indexD_neg[which.max(svm_line_D[indexD_neg,2])]
      lineCutOffD <- rbind(lineCutOffD,svm_line_D[indexD_neg_max,c(1:2)])
    }
    if(length(indexI)>0){
      indexI_neg <- indexI[svm_line_I[indexI,3]=="Negative"]
      indexI_neg_max <- indexI_neg[which.max(svm_line_I[indexI_neg,2])]
      lineCutOffI <- rbind(lineCutOffI,svm_line_I[indexI_neg_max,c(1:2)])
    }
  }
  svmList <- list()
  svmList["cutOffD"] <- list(lineCutOffD)
  svmList["cutOffI"] <- list(lineCutOffI)
  return(svmList)
}

#' Calculation of zeta and weighted zeta score.
#'
#' This function calculates zeta scores for genes based on their Z-score profiles across different thresholds. The zeta score quantifies the regulatory effect of gene knockdown on alternative splicing events by measuring the area under the curve of event coverage across Z-score thresholds.
#'
#' @param ZscoreVal A matrix of Z-scores where rows represent genes and columns represent readouts/conditions. This is typically the output from the Zscore() function.
#'
#' @param ZseqList A list containing two vectors: 'Zseq_D' (decrease direction thresholds) and 'Zseq_I' (increase direction thresholds). These define the Z-score bins for calculating event coverage.
#'
#' @param SVMcurve Optional. A matrix containing SVM curve data for decrease and increase directions. Required only when SVM=TRUE. The matrix should have 4 columns: Z-score and coverage for decrease direction (columns 1-2), and Z-score and coverage for increase direction (columns 3-4).
#'
#' @param SVM Logical. Whether to use SVM curves for background correction. Default is FALSE. When TRUE, the function subtracts SVM-predicted background from the event coverage before calculating zeta scores.
#'
#' @return A data frame with two columns:
#'   \item{Zeta_D}{Zeta score for decrease direction (exon skipping events)}
#'   \item{Zeta_I}{Zeta score for increase direction (exon inclusion events)}
#' 
#' Each row corresponds to a gene, and the zeta scores represent the cumulative regulatory effect across all Z-score thresholds.
#'
#' @details The function calculates zeta scores as follows:
#' \enumerate{
#'   \item For each Z-score threshold, calculates the proportion of readouts that exceed (increase) or fall below (decrease) the threshold
#'   \item Computes the area under the event coverage curve using trapezoidal integration
#'   \item If SVM=TRUE, subtracts SVM-predicted background coverage before area calculation
#'   \item Returns separate scores for decrease (Zeta_D) and increase (Zeta_I) directions
#' }
#' 
#' Higher zeta scores indicate stronger regulatory effects on alternative splicing.
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(ZseqList)
#' data(SVMcurve)
#' data(countMat)
#' data(negGene)
#' ZscoreVal <- Zscore(countMat, negGene)
#' zetaData <- Zeta(ZscoreVal, ZseqList, SVM=FALSE)
#'
#' @keywords ZetaSuite zeta score alternative splicing
#'
#' @name Zeta
#' @export Zeta

Zeta <- function(ZscoreVal,
                 ZseqList,
                 SVMcurve=NULL,
                 SVM=FALSE){

  Zseq_D <- ZseqList$Zseq_D
  Zseq_I <- ZseqList$Zseq_I
  ZscoreVal[is.na(ZscoreVal)] <- 0
  nColZ <- ncol(ZscoreVal)
  outputdata_D <- rep(0,nrow(ZscoreVal))
  outputdata_I <- rep(0,nrow(ZscoreVal))

  if(SVM==TRUE){
    if(is.null(SVMcurve)==TRUE){
      stop("SVMcurve should not be NULL when SVM is TRUE")
    }
    SVMcurveD <- SVMcurve[,1:2]
    SVMcurveI <- SVMcurve[,3:4]
    for (j in 1:(length(Zseq_D)-1)){
      lengthUse_D <- rowSums(ZscoreVal < Zseq_D[j])
      lengthUse_D_add <- rowSums(ZscoreVal < Zseq_D[j+1])
      conD <- (lengthUse_D/nColZ+lengthUse_D_add/nColZ-SVMcurveD[j,2]-SVMcurveD[j+1,2])*Zseq_D[j+1]*(Zseq_D[j]-Zseq_D[j+1])/2
      conD[conD < 0] <- 0
      outputdata_D<-outputdata_D+conD
      lengthUse_I <- rowSums(ZscoreVal > Zseq_I[j])
      lengthUse_I_add <- rowSums(ZscoreVal > Zseq_I[j+1])
      conI <- (lengthUse_I/nColZ+lengthUse_I_add/nColZ-SVMcurveI[j,2]-SVMcurveI[j+1,2])*Zseq_I[j+1]*(Zseq_I[j+1]-Zseq_I[j])/2
      conI[conI <0] <- 0
      outputdata_I <- outputdata_I + conI
    }
  }else{
    for (j in 1:(length(Zseq_D)-1)){
      lengthUse_D <- rowSums(ZscoreVal < Zseq_D[j])
      lengthUse_D_add <- rowSums(ZscoreVal < Zseq_D[j+1])
      outputdata_D <- outputdata_D + (lengthUse_D/nColZ+lengthUse_D_add/nColZ)*Zseq_D[j+1]*(Zseq_D[j]-Zseq_D[j+1])/2
      lengthUse_I <- rowSums(ZscoreVal > Zseq_I[j])
      lengthUse_I_add <- rowSums(ZscoreVal > Zseq_I[j+1])
      outputdata_I <- outputdata_I + (lengthUse_I/nColZ+lengthUse_I_add/nColZ)*Zseq_I[j+1]*(Zseq_I[j+1]-Zseq_I[j])/2
    }
  }
  outputdata_D <- data.frame(outputdata_D,stringsAsFactors = FALSE)
  colnames(outputdata_D)<-c("Zeta_D")
  outputdata_I <- data.frame(outputdata_I,stringsAsFactors = FALSE)
  colnames(outputdata_I)<-c("Zeta_I")

  zetaDat <- cbind(outputdata_D,outputdata_I)
  return(zetaDat)
}

#' Determine optimal cutoff thresholds based on Screen Strength analysis.
#'
#' This function calculates optimal cutoff thresholds for identifying significant hits in high-throughput screening data using Screen Strength (SS) analysis. It evaluates the trade-off between sensitivity and specificity by calculating the ratio of apparent FDR to baseline FDR across different zeta score thresholds.
#'
#' @param zetaData A data frame containing zeta scores calculated by the Zeta() function. Should have columns 'Zeta_D' and 'Zeta_I' representing decrease and increase direction scores, respectively.
#'
#' @param negGene A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in zetaData.
#'
#' @param posGene A data frame or matrix containing positive control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in zetaData.
#'
#' @param nonExpGene A data frame or matrix containing non-expressed gene/siRNA identifiers. These genes are used to estimate the baseline false discovery rate. The first column should contain gene/siRNA names that match the row names in zetaData.
#'
#' @param combine Logical. Whether to combine decrease and increase direction zeta scores. Default is FALSE. When TRUE, uses the sum of Zeta_D and Zeta_I; when FALSE, analyzes each direction separately.
#'
#' @return A list containing:
#'   \item{FDR_cutOff}{A data frame with 6 columns:
#'     \itemize{
#'       \item Cut_Off: Zeta score threshold
#'       \item aFDR: Apparent false discovery rate at this threshold
#'       \item SS: Screen Strength = 1 - (aFDR / bFDR)
#'       \item TotalHits: Total number of hits at this threshold
#'       \item Num_nonExp: Number of non-expressed genes among hits
#'       \item Type: Direction ("Decrease", "Increase", or "Combine")
#'     }
#'   }
#'   \item{plotList}{A list with two ggplot objects:
#'     \itemize{
#'       \item Zeta_type: Jitter plots showing zeta score distributions by gene type
#'       \item SS_cutOff: Screen Strength curves showing SS vs. zeta score threshold
#'     }
#'   }
#'
#' @details The function performs the following analysis:
#' \enumerate{
#'   \item Categorizes genes into types: "Gene" (test genes), "Positive" (positive controls), "NS_mix" (negative controls), and "non_exp" (non-expressed genes)
#'   \item Calculates baseline FDR (bFDR) as the proportion of non-expressed genes in the entire dataset
#'   \item For each zeta score threshold, calculates apparent FDR (aFDR) as the proportion of non-expressed genes among hits
#'   \item Computes Screen Strength: SS = 1 - (aFDR / bFDR)
#'   \item Generates plots showing zeta score distributions and SS curves
#' }
#' 
#' Higher Screen Strength values indicate better separation between true hits and false positives. Users can select appropriate thresholds based on desired sensitivity/specificity trade-offs.
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(nonExpGene)
#' data(negGene)
#' data(posGene)
#' data(ZseqList)
#' data(countMat)
#' ZscoreVal <- Zscore(countMat, negGene)
#' zetaData <- Zeta(ZscoreVal, ZseqList, SVM=FALSE)
#' cutoffval <- FDRcutoff(zetaData, negGene, posGene, nonExpGene, combine=TRUE)
#'
#' @keywords ZetaSuite FDR cutoff screen strength
#'
#' @importFrom ggplot2 ggplot geom_jitter theme_bw theme xlab scale_color_manual geom_point geom_smooth ylab aes_string element_blank
#'
#' @importFrom grDevices dev.off pdf
#'
#' @name FDRcutoff
#' @export FDRcutoff

FDRcutoff <- function (zetaData, negGene, posGene, nonExpGene, combine = FALSE) {
  zetaData$type <- rep("Gene", nrow(zetaData))
  zetaData[rownames(zetaData) %in% negGene[, 1], "type"] <- "NS_mix"
  zetaData[rownames(zetaData) %in% posGene[, 1], "type"] <- "Positive"
  zetaData[rownames(zetaData) %in% nonExpGene[, 1], "type"] <- "non_exp"
  nonGene_D <- zetaData[zetaData$type %in% c("non_exp", "Gene"), ]

  if (combine == FALSE) {
    maxD <- sort(nonGene_D[, "Zeta_D"], decreasing = TRUE)[10]
    maxI <- sort(nonGene_D[, "Zeta_I"], decreasing = TRUE)[10]
    minD <- sort(nonGene_D[, "Zeta_D"], decreasing = FALSE)[1]
    minI <- sort(nonGene_D[, "Zeta_I"], decreasing = FALSE)[1]
    stepD <- (maxD - minD)/100
    stepI <- (maxI - minI)/100
    sum1 <- nrow(nonGene_D)
    sum2 <- sum(nonGene_D[, "type"] %in% "non_exp")
    iFDR <- sum2/(sum1 - 1)
    seqD <- seq(minD, maxD, stepD)
    FDR_cutOff_de <- matrix(NA, length(seqD), 6)
    index <- 1
    for (num in seqD) {
      totalNum <- sum(zetaData[, "Zeta_D"] >= num & zetaData[, "type"] %in% c("non_exp", "Gene"))
      numNexp <- sum(zetaData[, "Zeta_D"] >= num & zetaData[, "type"] %in% c("non_exp"))
      FDR_Nexp <- numNexp/totalNum
      screen_Stress <- (iFDR - FDR_Nexp)/iFDR
      FDR_cutOff_de[index, ] <- c(num, FDR_Nexp,screen_Stress,totalNum, numNexp, "Decrease")
      index <- index + 1
    }
    seqI <- seq(minI, maxI, stepI)
    FDR_cutOff_in <- matrix(NA, length(seqI), 6)
    index <- 1
    for (num in seqI) {
      totalNum <- sum(zetaData[, "Zeta_I"] >= num & zetaData[, "type"] %in% c("non_exp", "Gene"))
      numNexp <- sum(zetaData[, "Zeta_I"] >= num & zetaData[, "type"] %in% c("non_exp"))
      FDR_Nexp <- numNexp/totalNum
      screen_Stress <- (iFDR - FDR_Nexp)/iFDR
      FDR_cutOff_in[index, ] <- c(num, FDR_Nexp, screen_Stress, totalNum, numNexp, "Increase")
      index <- index + 1
    }
    FDR_cutOff <- rbind(FDR_cutOff_de, FDR_cutOff_in)
  } else {
    maxD <- sort(nonGene_D[, 1] + nonGene_D[, 2], decreasing = TRUE)[10]
    minD <- sort(nonGene_D[, 1] + nonGene_D[, 2], decreasing = FALSE)[1]
    stepD <- (maxD - minD)/100
    sum1 <- nrow(nonGene_D)
    sum2 <- sum(nonGene_D[, 3] %in% "non_exp")
    iFDR <- sum2/(sum1 - 1)
    seqD <- seq(minD, maxD, stepD)
    FDR_cutOff <- matrix(NA, length(seqD), 6)
    index <- 1
    for (num in seqD) {
      totalNum <- sum(zetaData[, "Zeta_D"] + zetaData[, "Zeta_I"] >= num & zetaData[, "type"] %in% c("non_exp", "Gene"))
      numNexp <- sum(zetaData[, "Zeta_D"] + zetaData[, "Zeta_I"] >= num & zetaData[, "type"] %in% c("non_exp"))
      FDR_Nexp <- numNexp/totalNum
      screen_Stress <- (iFDR - FDR_Nexp)/iFDR
      FDR_cutOff[index, ] <- c(num, FDR_Nexp, screen_Stress, totalNum, numNexp, "Combine")
      index <- index + 1
    }
  }
  FDR_cutOff <- as.data.frame(FDR_cutOff,stringsAsFactors = FALSE)
  colnames(FDR_cutOff) <- c("Cut_Off","aFDR","SS","TotalHits","Num_nonExp","Type")
  FDR_cutOff$SS <- as.numeric(FDR_cutOff$SS)
  FDR_cutOff$Cut_Off <- as.numeric(FDR_cutOff$Cut_Off)
  zetaData_NS <- zetaData[zetaData$type != "NS_mix", ]

  p1 <- ggplot(zetaData_NS) + geom_jitter(aes_string(x = "type", y = "Zeta_D", col = "type")) + theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + xlab("") + scale_color_manual(values = c("#c994c7", "#67a9cf", "#ef8a62"))
  p2 <- ggplot(zetaData_NS) + geom_jitter(aes_string(x = "type", y = "Zeta_I", col = "type")) + theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + xlab("") + scale_color_manual(values = c("#c994c7", "#67a9cf", "#ef8a62"))
  p_Zeta_type <- gridExtra::grid.arrange(p1, p2, nrow = 1)

  if (combine == FALSE) {
    fdtss <- FDR_cutOff[FDR_cutOff$SS < 0.9, ]
    Dec <- fdtss[fdtss$Type == "Decrease", ]
    Inc <- fdtss[fdtss$Type == "Increase", ]

    p1 <- ggplot(Dec, aes_string(x = "Cut_Off", y = "SS", col = "Type")) + geom_point() + geom_smooth(span = 0.2) + theme_bw() + xlab("Zeta Score") + ylab("Screen strength") + theme(legend.position = c(0.8, 0.2), legend.title = element_blank())
    p2 <- ggplot(Inc, aes_string(x = "Cut_Off", y = "SS", col = "Type")) + geom_point() + geom_smooth(span = 0.2) + theme_bw() + xlab("Zeta Score") + ylab("Screen strength") + theme(legend.position = c(0.8, 0.2), legend.title = element_blank())
    p_SS_cutOff <- gridExtra::grid.arrange(p1, p2, nrow = 1)

  } else {
    fdtss <- FDR_cutOff[FDR_cutOff$SS < 1, ]

    p_SS_cutOff <- ggplot(fdtss, aes_string(x = "Cut_Off", y = "SS", col = "Type")) + geom_point() + geom_smooth(span = 0.2) + theme_bw() + xlab("Zeta Score") + ylab("Screen strength") + theme(legend.position = c(0.8, 0.2), legend.title = element_blank())
  }
  plotList <- list()
  plotList['Zeta_type'] <- list(p_Zeta_type)
  plotList['SS_cutOff'] <- list(p_SS_cutOff)
  return(list(FDR_cutOff,plotList))
}

#' Calculate zeta score for single cell RNA-seq quality control.
#'
#' This function evaluates the quality of cells detected in single-cell RNA-seq data by calculating a zeta score for each cell. The zeta score is based on the distribution of gene expression across different expression thresholds. A cutoff value is automatically determined using a two-component Gaussian mixture model to separate high-quality cells from low-quality or damaged cells.
#'
#' @param countMatSC A matrix of single-cell RNA-seq count data where rows represent cells and columns represent genes.
#'
#' @param binNum The number of bins for zeta score calculation. Default is 10. The function creates expression thresholds from 0 to the 80th percentile of non-zero expression values, divided into binNum intervals.
#'
#' @param filter Logical. Whether to filter out cells with total read counts less than 100. Default is TRUE. This helps remove extremely low-quality cells before analysis.
#'
#' @return A list containing:
#'   \item{zetaData}{A data frame with two columns: 'Cell' (cell identifiers) and 'Zeta' (calculated zeta scores)}
#'   \item{p_cutoff}{A ggplot object showing the distribution of log10-transformed zeta scores with fitted Gaussian mixture components and the determined cutoff threshold}
#'
#' @details The function works as follows:
#' \enumerate{
#'   \item Filters cells based on total read count if filter=TRUE
#'   \item Samples a subset of cells and genes for computational efficiency
#'   \item Creates expression thresholds (bins) from 0 to the 80th percentile of non-zero expression values
#'   \item For each cell, counts how many genes exceed each threshold
#'   \item Calculates the zeta score as a weighted sum of these counts
#'   \item Fits a two-component Gaussian mixture model to log10-transformed zeta scores
#'   \item Determines an optimal cutoff to separate high-quality from low-quality cells
#' }
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(countMatSC)
#' \donttest{zetaDataSC <- ZetaSuitSC(countMatSC, binNum=50, filter=TRUE)}
#'
#' @keywords ZetaSuite single cell quality control
#'
#' @importFrom ggplot2 ggplot geom_density geom_area xlim geom_vline theme_bw geom_text aes
#' @importFrom reshape2 melt
#' @importFrom mixtools normalmixEM
#'
#' @importFrom stats density dnorm
#'
#' @name ZetaSuitSC
#' @export ZetaSuitSC

ZetaSuitSC <- function(countMatSC,binNum=10,filter=TRUE){
  if(filter==TRUE){
    nCount <- which(rowSums(countMatSC)>100)
    inFilter <- countMatSC[nCount,]
  }else{
    inFilter <- countMatSC
  }
  #set.seed(1234)
  minvalue1 <- min(10000,floor(nrow(inFilter)*0.5))
  minvalue2 <- min(10000,floor(ncol(inFilter)*0.5))
  sampleVec <- inFilter[sample(nrow(inFilter),minvalue1),sample(ncol(inFilter),minvalue2)]
  Meltsample <- melt(sampleVec)
  MeltsamMt0 <- Meltsample[Meltsample$value>0,]
  maxVal <- round(sort(MeltsamMt0$value)[round(nrow(MeltsamMt0)*0.8)]/binNum)*binNum
  if(maxVal==0){
    Zseq <- seq(0,(binNum-1),1)
  }else{
    Zseq <- seq(0,maxVal,maxVal/binNum)
  }
  nFeatureDiffCutOff <- rownames(inFilter)
  for(i in Zseq){
    nFeatureDiffCutOff <- cbind(nFeatureDiffCutOff,data.frame(rowSums(inFilter > i),stringsAsFactors=FALSE))
  }
  colnames(nFeatureDiffCutOff) <- c("Cell",Zseq)
  if(binNum==10){
    binNum <- binNum - 1
  }
  allNum <- nFeatureDiffCutOff[,-1]
  zetaVal <- data.frame(rowSums(allNum[,2:binNum])+(allNum[,1]+allNum[,1+binNum])/2,stringsAsFactors=FALSE)
  zetaData <- cbind(nFeatureDiffCutOff[,1],zetaVal)
  colnames(zetaData) <- c("Cell","Zeta")

  #set.seed(1234)
  lowPop<-0.0001
  densityRes<-cbind(density(log10(zetaData$Zeta),bw=0.08)$x,density(log10(zetaData$Zeta),bw=0.08)$y)
  densityRes<-as.data.frame(densityRes,stringsAsFactors=FALSE)
  mid<-quantile(log10(zetaData$Zeta),probs=0.5)
  mean1<-densityRes[densityRes$V2==max(densityRes[densityRes$V1<mid,]$V2),]$V1
  mean2<-densityRes[densityRes$V2==max(densityRes[densityRes$V1>mid,]$V2),]$V1
  out<-normalmixEM(log10(zetaData$Zeta),k=2,epsilon = 1e-03,mean.constr=c(mean1,mean2))
  x <- seq(0,6,length.out = 1000)
  y1 <- dnorm(x, mean1,out$sigma[1])
  y2 <- dnorm(x, mean2,out$sigma[2])
  y1data<-as.data.frame(cbind(x,y1),stringsAsFactors=FALSE)
  cutoff<-round(10^round(y1data[y1data$y1<=lowPop & y1data$x>mean1 & y1data$x<mean2,][1,1],1),0)
  if(is.na(cutoff)){
    mid=quantile(log10(zetaData$Zeta),probs=0.1)
    mean1<-densityRes[densityRes$V2==max(densityRes[densityRes$V1<mid,]$V2),]$V1
    mean2<-densityRes[densityRes$V2==max(densityRes[densityRes$V1>mid,]$V2),]$V1
    out<-normalmixEM(log10(zetaData$Zeta),k=2,epsilon = 1e-03,mean.constr=c(mean1,mean2))
    x <- seq(0,6,length.out = 1000)
    y1 <- dnorm(x, mean1,out$sigma[1])
    y2 <- dnorm(x, mean2,out$sigma[2])
    y1data<-as.data.frame(cbind(x,y1),stringsAsFactors=FALSE)
    cutoff<-round(10^round(y1data[y1data$y1<=lowPop & y1data$x>mean1,][1,1],1),0)
    if(is.na(cutoff)){
      y2data<-as.data.frame(cbind(x,y2),stringsAsFactors=FALSE)
      cutoff<-round(10^round(y2data[y2data$y2>=lowPop,][1,1],1),0)
    }
  }
  string<-paste("cut-off is ", cutoff)
  p_cutoff <- ggplot()+geom_density(aes(log10(zetaData$Zeta)))+geom_area(aes(x=x,y=y1),fill="orange",alpha=0.3)+geom_area(aes(x=x,y=y2),fill="red",alpha=0.3)+xlim(0,6)+geom_vline(xintercept=log10(cutoff),linetype="dashed")+theme_bw()+geom_text(aes(x=log10(cutoff)+0.71, label=string, y=0.8))
  return(list(zetaData,p_cutoff))
}

#' Launch ZetaSuite Shiny Application
#'
#' @description
#' Launches the ZetaSuite Shiny web application for interactive analysis of 
#' high-throughput screening data and single-cell RNA-seq quality control.
#'
#' @details
#' The Shiny app provides a user-friendly interface for:
#' \itemize{
#'   \item Quality Control Analysis
#'   \item Z-score Normalization
#'   \item Event Coverage Analysis
#'   \item Zeta Score Calculation
#'   \item SVM-based Background Correction
#'   \item Screen Strength Analysis
#'   \item Single Cell Quality Control
#'   \item Interactive visualizations and data export
#' }
#'
#' @param launch.browser Logical. Should the app launch in the default browser?
#'   Default is TRUE.
#' @param port Integer. Port number for the Shiny app. Default is NULL (random port).
#' @param host Character. Host address. Default is "127.0.0.1" (localhost).
#'
#' @return Launches the Shiny application in a web browser.
#'
#' @examples
#' \dontrun{
#' # Launch the ZetaSuite Shiny app
#' ZetaSuiteApp()
#' 
#' # Launch without opening browser automatically
#' ZetaSuiteApp(launch.browser = FALSE)
#' 
#' # Launch on a specific port
#' ZetaSuiteApp(port = 3838)
#' }
#'
#' @importFrom shiny runApp tags h3 p br h4 code a fluidRow fileInput actionButton textOutput plotOutput numericInput checkboxInput downloadButton verbatimTextOutput tabsetPanel tabPanel reactiveValues observeEvent req showNotification renderPlot
#' @importFrom shinydashboard dashboardPage dashboardHeader dashboardSidebar sidebarMenu menuItem box tabItems tabItem
#' @importFrom DT datatable renderDataTable dataTableOutput
#' @importFrom plotly ggplotly
#' @importFrom shinyjs useShinyjs
#' @export
ZetaSuiteApp <- function(launch.browser = TRUE, port = NULL, host = "127.0.0.1") {
  
  # Check if required packages are installed (suggested dependencies)
  required_packages <- c("shiny", "shinydashboard", "DT", "plotly", "shinyjs")
  missing_packages <- required_packages[!sapply(required_packages, requireNamespace, quietly = TRUE)]
  
  if (length(missing_packages) > 0) {
    stop(
      paste0(
        "The following packages are required to run the ZetaSuite Shiny app but are not installed:\n",
        paste(missing_packages, collapse = ", "),
        "\n\nPlease install them using:\n",
        "install.packages(c(", paste0("'", missing_packages, "'", collapse = ", "), "))"
      ),
      call. = FALSE
    )
  }
  
  # Resolve path to the embedded Shiny app
  app_path <- system.file("shiny", package = "ZetaSuite")
  if (isTRUE(nchar(app_path) == 0L) || app_path == "") {
    stop("Shiny app files not found. Please ensure the ZetaSuite package is properly installed.", call. = FALSE)
  }
  
  # Launch the Shiny app without attaching packages
  shiny::runApp(
    appDir = app_path,
    launch.browser = launch.browser,
    port = port,
    host = host
  )
} 

#' Z-score normalization for high-throughput screening data.
#'
#' This function performs Z-score normalization on high-throughput screening data using negative control samples as reference. The Z-score transformation standardizes the data by centering and scaling each column (readout) based on the mean and standard deviation of negative control samples.
#' 
#' @param countMat A matrix of raw count data where rows represent genes/siRNAs and columns represent readouts/conditions. The matrix should have row names corresponding to gene/siRNA identifiers.
#'
#' @param negGene A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain the gene/siRNA names that match the row names in countMat.
#'
#' @details The function performs Z-score normalization as follows:
#' \enumerate{
#'   \item Extracts negative control samples from the input matrix using the identifiers provided in negGene
#'   \item For each column (readout), calculates the mean and standard deviation using only the negative control samples
#'   \item Applies Z-score transformation: \eqn{Z_{ij} = (X_{ij} - \mu_{j}) / \sigma_{j}}
#'   where \eqn{X_{ij}} is the raw value for gene \eqn{i} in readout \eqn{j}, \eqn{\mu_{j}} is the mean of negative controls in readout \eqn{j}, and \eqn{\sigma_{j}} is the standard deviation of negative controls in readout \eqn{j}
#' }
#' 
#' This normalization allows for comparison across different readouts and identifies genes/siRNAs that show significant deviation from the negative control distribution.
#' 
#' @return A Z-score normalized matrix with the same dimensions as the input countMat (excluding the Type column added during processing). Each value represents how many standard deviations away from the negative control mean that particular gene/readout combination is.
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(countMat)
#' data(negGene)
#' ZscoreVal <- Zscore(countMat, negGene)
#' ZscoreVal[1:5, 1:5]
#'
#' @keywords ZetaSuite normalization Z-score
#'
#' @importFrom stats sd
#'
#' @name Zscore
#' @export Zscore

Zscore <- function (countMat, negGene) {
    countMatNeg <- countMat[rownames(countMat) %in% negGene[, 1], ]
    countMatNeg$Type <- rep("Negative", nrow(countMatNeg))
    Nmix <- countMatNeg
    rownames(Nmix) <- rownames(countMatNeg)
    matrixData <- countMat
    rownames(matrixData) <- rownames(countMat)
    outputdata <- matrix(0, nrow(matrixData), ncol(Nmix) - 1)
    for (i in 1:(ncol(Nmix) - 1)) {
        outputdata[, i] <- (((matrixData[, i]) - mean(Nmix[, i]))/sd(Nmix[, i]))
    }
    colnames(outputdata) <- colnames(Nmix)[-ncol(Nmix)]
    rownames(outputdata) <- rownames(matrixData)
    return(outputdata)
}

#' Generate event coverage analysis and visualization for alternative splicing data.
#'
#' This function analyzes event coverage across Z-score thresholds and generates visualizations to compare positive and negative control samples. It calculates the proportion of readouts that exceed different Z-score thresholds for each gene, creating the foundation for zeta score calculations.
#'
#' @param ZscoreVal A matrix of Z-scores where rows represent genes and columns represent readouts/conditions. This is typically the output from the Zscore() function.
#'
#' @param negGene A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in ZscoreVal.
#'
#' @param posGene A data frame or matrix containing positive control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in ZscoreVal.
#'
#' @param binNum The number of bins to divide the Z-score range. Recommended value is 100. The function creates Z-score thresholds from the 0.00001 to 0.99999 quantiles of the data.
#'
#' @param combine Logical. Whether to combine the negative and positive Z-score ranges. Default is TRUE. When TRUE, uses symmetric ranges around zero; when FALSE, uses separate ranges for negative and positive values.
#'
#' @return A list containing two sublists:
#'   \item{ECdataList}{A list with the following components:
#'     \itemize{
#'       \item ZseqList: A data frame with Z-score thresholds for decrease (Zseq_D) and increase (Zseq_I) directions
#'       \item EC_N_I: Event coverage matrix for negative controls in increase direction
#'       \item EC_N_D: Event coverage matrix for negative controls in decrease direction
#'       \item EC_P_I: Event coverage matrix for positive controls in increase direction
#'       \item EC_P_D: Event coverage matrix for positive controls in decrease direction
#'     }
#'   }
#'   \item{ECplotList}{A list with two ggplot objects:
#'     \itemize{
#'       \item EC_jitter_D: Jitter plot showing event coverage for decrease direction
#'       \item EC_jitter_I: Jitter plot showing event coverage for increase direction
#'     }
#'   }
#'
#' @details The function performs the following steps:
#' \enumerate{
#'   \item Determines Z-score thresholds based on data quantiles and binNum
#'   \item For each gene and threshold, calculates the proportion of readouts that exceed (increase) or fall below (decrease) the threshold
#'   \item Separates data into negative and positive control groups
#'   \item Generates jitter plots comparing event coverage between control groups
#'   \item Returns both data matrices and visualization plots
#' }
#' 
#' The event coverage matrices can be used as input for SVM analysis and zeta score calculations.
#'
#' @author Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
#'
#' @examples
#' data(countMat)
#' data(negGene)
#' data(posGene)
#' ZscoreVal <- Zscore(countMat, negGene)
#' ECList <- EventCoverage(ZscoreVal, negGene, posGene, binNum=100, combine=TRUE)
#'
#' @keywords ZetaSuite event coverage alternative splicing
#'
#' @importFrom ggplot2 ggplot geom_jitter ylab xlab theme theme_bw scale_fill_manual aes element_text element_blank
#' @importFrom reshape2 melt
#' @importFrom RColorBrewer brewer.pal
#'
#' @importFrom grDevices colorRampPalette dev.off pdf png
#'
#' @importFrom stats quantile
#'
#' @name EventCoverage
#' @export EventCoverage

EventCoverage <- function (ZscoreVal, negGene, posGene, binNum, combine = TRUE){
    ZscoreVal[is.na(ZscoreVal)] <- 0
	old <- options()
	on.exit(options(old))
    options(digits = 15)
    meltdata <- melt(ZscoreVal)
    if (binNum <= 0){
        stop("binNum should be more than 0")
    }
    if (combine == TRUE){
        minV = quantile(meltdata$value, probs = c(1e-05))
        maxV = quantile(meltdata$value, probs = c(0.99999))
        maxValAbs <- max(abs(minV), abs(maxV))
        stepmax = maxValAbs/binNum
        Zseq_D <- seq(maxValAbs * (-1), -1.3, abs(stepmax))
        Zseq_I <- seq(1.3, maxValAbs, abs(stepmax))
    }else {
        minV = quantile(meltdata$value, probs = c(1e-05))
        stepmin = minV/binNum
        Zseq_D <- seq(minV, 0, abs(stepmin))
        maxV = quantile(meltdata$value, probs = c(0.99999))
        stepmax = maxV/binNum
        Zseq_I <- seq(0, maxV, abs(stepmax))
    }
    ZseqDI <- data.frame(Zseq_D, Zseq_I,stringsAsFactors = FALSE)

    nColCM <- ncol(ZscoreVal)
    EC_D <- matrix(NA, nrow(ZscoreVal), length(Zseq_D))
    colnames(EC_D) <- Zseq_D
    rownames(EC_D) <- rownames(ZscoreVal)
    EC_I <- EC_D
    colnames(EC_I) <- Zseq_I
    for (i in 1:length(Zseq_D)){
        EC_D[, i] <- rowSums(ZscoreVal < Zseq_D[i])/nColCM
        EC_I[, i] <- rowSums(ZscoreVal > Zseq_I[i])/nColCM
    }

    EC_I_N <- EC_I[rownames(EC_I) %in% negGene[, 1], ]
    EC_D_N <- EC_D[rownames(EC_D) %in% negGene[, 1], ]
    EC_I_P <- EC_I[rownames(EC_I) %in% posGene[, 1], ]
    EC_D_P <- EC_D[rownames(EC_D) %in% posGene[, 1], ]
    ECDN <- melt(EC_D_N)
    ECDP <- melt(EC_D_P)
    ECIN <- melt(EC_I_N)
    ECIP <- melt(EC_I_P)

    getPalette = colorRampPalette(brewer.pal(9, "Set1"))
    p_EC_jitter_D <- ggplot() + geom_jitter(aes(x = ECDN[, 2], y = ECDN[, 3]), colour = "#67a9cf", size = 0.1) + geom_jitter(aes(x = ECDP[, 2], y = ECDP[, 3]), colour = "#ef8a62", size = 0.1) + ylab("Events percent") + xlab("Zscore") + theme(axis.text.x = element_text(angle = 60, hjust = 1, vjust = 1), panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + scale_fill_manual(values = getPalette(22)) + theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank())
    getPalette = colorRampPalette(brewer.pal(9, "Set1"))
    p_EC_jitter_I <- ggplot() + geom_jitter(aes(x = ECIN[, 2], y = ECIN[, 3]), colour = "#67a9cf", size = 0.1) + geom_jitter(aes(x = ECIP[, 2], y = ECIP[,3]), colour = "#ef8a62", size = 0.1) + ylab("Events percent") + xlab("Zscore") + theme(axis.text.x = element_text(angle = 60, hjust = 1, vjust = 1), panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + scale_fill_manual(values = getPalette(22)) + theme_bw() + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank())
    ECdataList <- list()
    ECdataList["ZseqList"] <- list(ZseqDI)
    ECdataList["EC_N_I"] <- list(EC_I_N)
    ECdataList["EC_N_D"] <- list(EC_D_N)
    ECdataList["EC_P_I"] <- list(EC_I_P)
    ECdataList["EC_P_D"] <- list(EC_D_P)
    ECplotList <- list()
    ECplotList['EC_jitter_D'] <- list(p_EC_jitter_D)
    ECplotList['EC_jitter_I'] <- list(p_EC_jitter_I)
    return(list(ECdataList,ECplotList))
}

---
title: "ZetaSuite: A Comprehensive Guide to Multi-dimensional High-throughput Data Analysis"
author: "Yajing Hao, Shuyang Zhang, Junhui Li, Changwei Shao, Guofeng Zhao, Xiang-Dong Fu"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
keep_html: true
vignette: >
  %\VignetteIndexEntry{ZetaSuite}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE} 
knitr::opts_chunk$set(warning = FALSE, message = FALSE, fig.width = 8, fig.height = 6) 
```

# Introduction

## Overview

ZetaSuite is an R package designed for analyzing multi-dimensional high-throughput screening data, particularly two-dimensional RNAi screens and single-cell RNA sequencing data. The package addresses the limitations of simple Z-based statistics when dealing with complex multi-dimensional datasets where experimental noise and off-target effects accumulate.

## Key Features

- **Quality Control Analysis**: Comprehensive evaluation of experimental design and data quality
- **Z-score Normalization**: Standardization using negative controls as reference
- **Event Coverage Analysis**: Quantification of regulatory effects across thresholds
- **Zeta Score Calculation**: Area-under-curve based scoring for regulatory effects
- **SVM-based Background Correction**: Machine learning approach to filter noise
- **Screen Strength Analysis**: Optimal threshold selection for hit identification
- **Single Cell Quality Control**: Cell quality assessment for scRNA-seq data

## Applications

1. **Two-dimensional RNAi screens**: Identify genes critical for cell fitness and proliferation
2. **Alternative splicing analysis**: Detect splicing regulators and their effects
3. **Single-cell RNA-seq QC**: Differentiate high-quality cells from damaged ones
4. **Cancer dependency screens**: Reveal genes with opposite roles in cancer biology

# Installation and Setup

## R Package Installation

```{r install, eval=FALSE}
# Install from CRAN
install.packages("ZetaSuite")

# Load the package
library(ZetaSuite)
```

## Interactive Shiny Application

ZetaSuite includes an interactive web interface for easy-to-use analysis:

```{r shiny_app, eval=FALSE}
# Launch the Shiny app
ZetaSuiteApp()

# Launch without opening browser automatically
ZetaSuiteApp(launch.browser = FALSE)

# Launch on a specific port
ZetaSuiteApp(port = 3838)
```

The Shiny app provides:
- **Interactive data upload** and visualization
- **Step-by-step analysis workflow** with progress indicators
- **Real-time results** and interactive plots
- **Data export capabilities** for all analysis results
- **Built-in example dataset** for demonstration
- **Bug report integration** with GitHub issues

## Original ZetaSuite Installation

# Example Dataset

The package includes an example dataset from an in-house HTS2 screening experiment. This dataset contains:

- **countMat**: 1,609 genes × 100 alternative splicing events (fold change values)
- **negGene**: 30 negative control genes (non-specific siRNAs)
- **posGene**: 20 positive control genes (PTB-targeting siRNAs)
- **nonExpGene**: 50 non-expressed genes (RPKM < 1 in HeLa cells)
- **ZseqList**: Pre-calculated Z-score thresholds
- **SVMcurve**: Pre-calculated SVM decision boundaries

```{r load_data}
library(ZetaSuite)

# Load example data
data(countMat)
data(negGene)
data(posGene)
data(nonExpGene)
data(ZseqList)
data(SVMcurve)

# Display data dimensions
cat("Count matrix dimensions:", dim(countMat), "\n")
cat("Negative controls:", nrow(negGene), "genes\n")
cat("Positive controls:", nrow(posGene), "genes\n")
cat("Non-expressed genes:", nrow(nonExpGene), "genes\n")
```

# Analysis Workflow

## Step 1: Quality Control Analysis

Quality control evaluates the ability of functional readouts to discriminate between negative and positive controls. This step provides diagnostic plots and SSMD (Strictly Standardized Mean Difference) scores.

```{r qc_analysis}
# Perform quality control analysis
qc_results <- QC(countMat, negGene, posGene)

# Display QC plots
cat("QC analysis completed. Generated", length(qc_results), "diagnostic plots.\n")
```

### QC Plot 1: Score Distribution
This plot shows the distribution of raw scores across all readouts for positive and negative controls.

```{r qc_plot1, fig.height = 5, fig.width = 8}
qc_results$score_qc
```

### QC Plot 2: t-SNE Visualization
Global evaluation of sample separation based on all readouts.

```{r qc_plot2, fig.height = 5, fig.width = 6}
qc_results$tSNE_QC
```

### QC Plot 3: Box Plots
Side-by-side comparison of score distributions between control groups.

```{r qc_plot3, fig.height = 4, fig.width = 8}
grid::grid.draw(qc_results$QC_box)
```

### QC Plot 4: SSMD Distribution
Distribution of SSMD scores with quality threshold (SSMD ≥ 2).

```{r qc_plot4, fig.height = 5, fig.width = 6}
qc_results$QC_SSMD
```

## Step 2: Z-score Normalization

Z-score normalization standardizes the data using negative controls as reference, making readouts comparable across different conditions.

```{r zscore_analysis}
# Calculate Z-scores
zscore_matrix <- Zscore(countMat, negGene)

# Display first few rows and columns
cat("Z-score matrix dimensions:", dim(zscore_matrix), "\n")
cat("First 5 rows and columns:\n")
print(zscore_matrix[1:5, 1:5])
```

## Step 3: Event Coverage Analysis

Event coverage quantifies the proportion of readouts that exceed different Z-score thresholds for each gene, creating the foundation for zeta score calculations.

```{r event_coverage}
# Calculate event coverage
ec_results <- EventCoverage(zscore_matrix, negGene, posGene, binNum = 100, combine = TRUE)

# Display event coverage plots
cat("Event coverage analysis completed.\n")
```

### Event Coverage Plots

```{r ec_plots, fig.height = 5, fig.width = 10}
# Decrease direction (exon skipping)
ec_results[[2]]$EC_jitter_D
```

```{r ec_plots2, fig.height = 5, fig.width = 10}
# Increase direction (exon inclusion)
ec_results[[2]]$EC_jitter_I
```

## Step 4: Zeta Score Calculation

Zeta scores represent the area under the event coverage curve, quantifying the cumulative regulatory effect of each gene across all Z-score thresholds.

```{r zeta_calculation}
# Calculate zeta scores without SVM correction
zeta_scores <- Zeta(zscore_matrix, ZseqList, SVM = FALSE)

# Display summary statistics
cat("Zeta score summary:\n")
cat("Number of genes:", nrow(zeta_scores), "\n")
cat("Zeta_D range:", range(zeta_scores$Zeta_D), "\n")
cat("Zeta_I range:", range(zeta_scores$Zeta_I), "\n")

# Show top hits
cat("\nTop 10 genes by Zeta_D (decrease direction):\n")
top_decrease <- head(zeta_scores[order(zeta_scores$Zeta_D, decreasing = TRUE), ], 10)
print(top_decrease)
```

## Step 5: SVM Background Correction (Optional)

SVM analysis creates decision boundaries to separate positive and negative controls, enabling background correction in zeta score calculations.

```{r svm_analysis, eval=FALSE}
# Run SVM analysis (can be computationally intensive)
svm_results <- SVM(ec_results)

# Calculate zeta scores with SVM correction
zeta_scores_svm <- Zeta(zscore_matrix, ZseqList, SVMcurve = svm_results, SVM = TRUE)
```

## Step 6: Screen Strength Analysis

Screen Strength analysis determines optimal cutoff thresholds by balancing sensitivity and specificity, using the ratio of apparent FDR to baseline FDR.

```{r screen_strength}
# Calculate FDR cutoffs and Screen Strength
fdr_results <- FDRcutoff(zeta_scores, negGene, posGene, nonExpGene, combine = TRUE)

# Display Screen Strength plots
cat("Screen Strength analysis completed.\n")
```

### Zeta Score Distribution by Gene Type

```{r ss_plot1, fig.height = 5, fig.width = 8}
fdr_results[[2]]$Zeta_type
```

### Screen Strength Curves

```{r ss_plot2, fig.height = 5, fig.width = 6}
fdr_results[[2]]$SS_cutOff
```

### FDR Cutoff Results

```{r fdr_table}
# Display FDR cutoff results
fdr_table <- fdr_results[[1]]
cat("FDR cutoff results summary:\n")
cat("Number of thresholds tested:", nrow(fdr_table), "\n")
cat("Screen Strength range:", range(fdr_table$SS), "\n")

# Show optimal thresholds (SS > 0.8)
optimal_thresholds <- fdr_table[fdr_table$SS > 0.8, ]
cat("\nOptimal thresholds (SS > 0.8):\n")
print(head(optimal_thresholds, 10))
```

## Step 7: Hit Selection

Based on the Screen Strength analysis, select appropriate thresholds for hit identification.

```{r hit_selection}
# Example: Select threshold with SS > 0.8 and reasonable number of hits
selected_threshold <- fdr_table[fdr_table$SS > 0.8 & fdr_table$TotalHits > 50, ]
if(nrow(selected_threshold) > 0) {
  best_threshold <- selected_threshold[which.max(selected_threshold$SS), ]
  cat("Recommended threshold:", best_threshold$Cut_Off, "\n")
  cat("Screen Strength:", best_threshold$SS, "\n")
  cat("Total hits:", best_threshold$TotalHits, "\n")
  
  # Identify hits
  combined_zeta <- zeta_scores$Zeta_D + zeta_scores$Zeta_I
  hits <- names(combined_zeta[combined_zeta >= best_threshold$Cut_Off])
  cat("Number of hits identified:", length(hits), "\n")
}
```

# Single Cell RNA-seq Quality Control

ZetaSuite also provides functionality for single-cell RNA-seq quality control, helping to differentiate high-quality cells from damaged ones.

```{r single_cell_example, eval=FALSE}
# Example single cell analysis (requires single cell count matrix)
# single_cell_results <- ZetaSuitSC(count_matrix_sc, binNum = 10, filter = TRUE)
```

# Advanced Usage

## Custom Parameter Settings

```{r custom_params, eval=FALSE}
# Custom event coverage analysis
ec_custom <- EventCoverage(zscore_matrix, negGene, posGene, 
                          binNum = 200,    # More bins for finer resolution
                          combine = FALSE) # Separate decrease/increase directions

# Custom zeta score calculation with SVM
zeta_custom <- Zeta(zscore_matrix, ZseqList, 
                   SVMcurve = svm_results, 
                   SVM = TRUE)  # Use SVM correction

# Custom FDR analysis
fdr_custom <- FDRcutoff(zeta_scores, negGene, posGene, nonExpGene, 
                       combine = FALSE)  # Analyze directions separately
```

## Batch Processing

```{r batch_processing, eval=FALSE}
# Example: Process multiple datasets
datasets <- list(dataset1 = list(countMat = countMat1, negGene = negGene1),
                dataset2 = list(countMat = countMat2, negGene = negGene2))

results <- lapply(datasets, function(ds) {
  zscore <- Zscore(ds$countMat, ds$negGene)
  ec <- EventCoverage(zscore, ds$negGene, ds$posGene, binNum = 100)
  zeta <- Zeta(zscore, ZseqList, SVM = FALSE)
  return(list(zscore = zscore, ec = ec, zeta = zeta))
})
```

# Interpretation Guidelines

## Quality Control Interpretation

- **SSMD ≥ 2**: High-quality readouts with good separation between controls
- **t-SNE separation**: Clear clustering indicates good experimental design
- **Box plot overlap**: Minimal overlap suggests effective control design

## Zeta Score Interpretation

- **Higher Zeta_D**: Genes promoting exon skipping
- **Higher Zeta_I**: Genes promoting exon inclusion
- **Combined score**: Overall regulatory effect strength

## Screen Strength Interpretation

- **SS = 1**: Perfect separation (no false positives)
- **SS > 0.8**: Excellent separation
- **SS > 0.6**: Good separation
- **Optimal threshold**: Balance between sensitivity and specificity

# Troubleshooting

## Common Issues

1. **Insufficient controls**: Ensure adequate number of positive/negative controls
2. **Data quality**: Check for missing values and extreme outliers
3. **Memory issues**: Reduce bin number for large datasets
4. **SVM convergence**: Adjust SVM parameters or use pre-calculated curves

## Performance Optimization

- Use `combine = TRUE` for faster event coverage analysis
- Reduce `binNum` for large datasets
- Consider using pre-calculated SVM curves for repeated analyses

# References

## Software Citation
Hao, Y., Shao, C., Zhao, G., Fu, X.D. (2021). ZetaSuite: A Computational Method for Analyzing Multi-dimensional High-throughput Data, Reveals Genes with Opposite Roles in Cancer Dependency. *Forthcoming*

## Dataset Citation
Shao, C., Hao, Y., Qiu, J., Zhou, B., Li, H., Zhou, Y., Meng, F., Jiang, L., Gou, L.T., Xu, J., Li, Y., Wang, H., Yeo, G.W., Wang, D., Ji, X., Glass, C.K., Aza-Blanc, P., Fu, X.D. (2021). HTS2 Screen for Global Splicing Regulators Reveals a Key Role of the Pol II Subunit RPB9 in Coupling between Transcription and Pre-mRNA Splicing. *Cell. Forthcoming*

# Session Information

```{r sessionInfo}
sessionInfo()
``` 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EventCoverage.R
\name{EventCoverage}
\alias{EventCoverage}
\title{Generate event coverage analysis and visualization for alternative splicing data.}
\usage{
EventCoverage(ZscoreVal, negGene, posGene, binNum, combine = TRUE)
}
\arguments{
\item{ZscoreVal}{A matrix of Z-scores where rows represent genes and columns represent readouts/conditions. This is typically the output from the Zscore() function.}

\item{negGene}{A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in ZscoreVal.}

\item{posGene}{A data frame or matrix containing positive control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in ZscoreVal.}

\item{binNum}{The number of bins to divide the Z-score range. Recommended value is 100. The function creates Z-score thresholds from the 0.00001 to 0.99999 quantiles of the data.}

\item{combine}{Logical. Whether to combine the negative and positive Z-score ranges. Default is TRUE. When TRUE, uses symmetric ranges around zero; when FALSE, uses separate ranges for negative and positive values.}
}
\value{
A list containing two sublists:
  \item{ECdataList}{A list with the following components:
    \itemize{
      \item ZseqList: A data frame with Z-score thresholds for decrease (Zseq_D) and increase (Zseq_I) directions
      \item EC_N_I: Event coverage matrix for negative controls in increase direction
      \item EC_N_D: Event coverage matrix for negative controls in decrease direction
      \item EC_P_I: Event coverage matrix for positive controls in increase direction
      \item EC_P_D: Event coverage matrix for positive controls in decrease direction
    }
  }
  \item{ECplotList}{A list with two ggplot objects:
    \itemize{
      \item EC_jitter_D: Jitter plot showing event coverage for decrease direction
      \item EC_jitter_I: Jitter plot showing event coverage for increase direction
    }
  }
}
\description{
This function analyzes event coverage across Z-score thresholds and generates visualizations to compare positive and negative control samples. It calculates the proportion of readouts that exceed different Z-score thresholds for each gene, creating the foundation for zeta score calculations.
}
\details{
The function performs the following steps:
\enumerate{
  \item Determines Z-score thresholds based on data quantiles and binNum
  \item For each gene and threshold, calculates the proportion of readouts that exceed (increase) or fall below (decrease) the threshold
  \item Separates data into negative and positive control groups
  \item Generates jitter plots comparing event coverage between control groups
  \item Returns both data matrices and visualization plots
}

The event coverage matrices can be used as input for SVM analysis and zeta score calculations.
}
\examples{
data(countMat)
data(negGene)
data(posGene)
ZscoreVal <- Zscore(countMat, negGene)
ECList <- EventCoverage(ZscoreVal, negGene, posGene, binNum=100, combine=TRUE)

}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{ZetaSuite}
\keyword{alternative}
\keyword{coverage}
\keyword{event}
\keyword{splicing}

\name{ZseqList}
\alias{ZseqList}
\docType{data}
\title{
  The bin size for Zeta calculation. 
  }
\description{
  A data frame with 11 different cut-offs and 2 directions. We divided the ranges of input values into bins. The number of bins is determined by the users.

  }
\usage{data("ZseqList")}
\format{
  A data frame with 11 different cut-offs and 2 directions.
  \describe{
    A data frame with 11 different cut-offs and 2 directions.We divided the ranges of input values into bins. The number of bins is determined by the users.
  }
}
\details{
  This data frame is the generated by EventCoverage.R.
}

\examples{
  data(ZseqList)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZetaSuiteApp.R
\name{ZetaSuiteApp}
\alias{ZetaSuiteApp}
\title{Launch ZetaSuite Shiny Application}
\usage{
ZetaSuiteApp(launch.browser = TRUE, port = NULL, host = "127.0.0.1")
}
\arguments{
\item{launch.browser}{Logical. Should the app launch in the default browser?
Default is TRUE.}

\item{port}{Integer. Port number for the Shiny app. Default is NULL (random port).}

\item{host}{Character. Host address. Default is "127.0.0.1" (localhost).}
}
\value{
Launches the Shiny application in a web browser.
}
\description{
Launches the ZetaSuite Shiny web application for interactive analysis of 
high-throughput screening data and single-cell RNA-seq quality control.
}
\details{
The Shiny app provides a user-friendly interface for:
\itemize{
  \item Quality Control Analysis
  \item Z-score Normalization
  \item Event Coverage Analysis
  \item Zeta Score Calculation
  \item SVM-based Background Correction
  \item Screen Strength Analysis
  \item Single Cell Quality Control
  \item Interactive visualizations and data export
}
}
\examples{
\dontrun{
# Launch the ZetaSuite Shiny app
ZetaSuiteApp()

# Launch without opening browser automatically
ZetaSuiteApp(launch.browser = FALSE)

# Launch on a specific port
ZetaSuiteApp(port = 3838)
}

}

\name{nonExpGene}
\alias{nonExpGene}
\docType{data}
\title{
  Input internal negative control file. 
  }
\description{
  A data frame with 722 different well IDs in which the cells treated with siRNAs targeting to non-expressed genes in HeLa cells.It the subset of total non-expressed 
  genes in HeLa cells.
  }
\usage{data("nonExpGene")}
\format{
  A data frame with 722 different well IDs in which the cells treated with siRNAs targeting to non-expressed genes in HeLa cells.
  \describe{
    A data frame with 722 different well IDs in which the cells treated with siRNAs targeting to non-expressed genes in HeLa cells. These wells were served as internal negative controls.
  }
}
\details{
  These non-expressed genes can be obtained from a prior expression profile.
}

\examples{
  data(nonExpGene)
}
\keyword{datasets}

\name{negGene}
\alias{negGene}
\docType{data}
\title{
  Input negative file. 
  }
\description{
  A data frame with 510 different well IDs in which the cells treated with non-specific siRNAs.If users did not have the build-in negative controls, the non-expressed genes should be provided here.
  }
\usage{data("negGene")}
\format{
  A data frame with 510 different well IDs in which the cells treated with non-specific siRNAs.
  \describe{
    A data frame with 510 different well IDs in which the cells treated with non-specific siRNAs. These wells were served as negative control.
  }
}
\details{
  These wells were designed by the authors in the large-scale screen.
}

\examples{
  data(negGene)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QC.R
\name{QC}
\alias{QC}
\title{Perform quality control analysis for high-throughput screening data.}
\usage{
QC(countMat, negGene, posGene)
}
\arguments{
\item{countMat}{A matrix of raw count data where rows represent genes/siRNAs and columns represent readouts/conditions. The matrix should have row names corresponding to gene/siRNA identifiers.}

\item{negGene}{A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in countMat.}

\item{posGene}{A data frame or matrix containing positive control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in countMat.}
}
\value{
A list containing four diagnostic plots:
  \item{score_qc}{A jitter plot showing the distribution of raw scores across all readouts for positive and negative controls}
  \item{tSNE_QC}{A t-SNE plot showing the global separation of positive and negative control samples in 2D space}
  \item{QC_box}{Side-by-side boxplots showing the distribution of scores for positive and negative controls across all readouts}
  \item{QC_SSMD}{A density plot showing the distribution of SSMD scores across readouts, with a threshold line at SSMD=2 and the percentage of high-quality readouts displayed}
}
\description{
This function performs comprehensive quality control analysis on high-throughput screening data to evaluate experimental design and data quality. It generates multiple diagnostic plots and calculates SSMD (Strictly Standardized Mean Difference) scores to assess the separation between positive and negative controls.
}
\details{
The function performs the following quality control analyses:
\enumerate{
  \item Creates jitter plots to visualize score distributions across readouts
  \item Performs t-SNE dimensionality reduction to assess global sample separation
  \item Generates boxplots to compare score distributions between control groups
  \item Calculates SSMD scores for each readout: \eqn{\mathrm{SSMD} = (\mu_{pos} - \mu_{neg}) / \sqrt{\sigma_{pos}^2 + \sigma_{neg}^2}}
  \item Reports the percentage of readouts with \eqn{|\mathrm{SSMD}| \ge 2} (considered high quality)
}

SSMD scores \eqn{\ge 2} indicate good separation between positive and negative controls, suggesting high-quality readouts.
}
\examples{
data(countMat)
data(negGene)
data(posGene)
\donttest{QC(countMat, negGene, posGene)}

}
\references{
Laurens van der Maaten & Geoffrey Hinton: Visualizing Data using t-SNE. Journal of Machine Learning Research 2008, 9(2008):2579-2605.

Zhang XD: A pair of new statistical parameters for quality control in RNA interference high-throughput screening assays. Genomics 2007, 89:552-561.
}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{SSMD}
\keyword{ZetaSuite}
\keyword{control}
\keyword{quality}
\keyword{t-SNE}

\name{countMatSC}
\alias{countMatSC}
\docType{data}
\title{
  The cell x gene matrix from single-cell RNA-seq. 
}
\description{
  A scRNA-seq dataset generated from placenta that has been analyzed with CellRanger and used to develop EmptyDrops. We have subsampled the genes from the real datasets to generated the matrix. 
}
\usage{data("countMatSC")}
\format{
  A data frame with 1090 cells and 10000 genes. This is the subset of data obtained from single-cell RNAseq for package testing.  Each row represents one cell detected in single-cell RNA-seq, each column is one gene in detected cells. The values in the matrix are the raw read counts from single-cell RNAseq. 
  \describe{
    A data frame with 1090 cells and 10000 genes.This is the subset of data obtained from single-cell RNAseq for package testing.
  }
}
\details{
  This data frame is the generated by single-cell RNA-seq.
}

\examples{
  data(countMatSC)
}
\keyword{datasets}

\name{posGene}
\alias{posGene}
\docType{data}
\title{
  Input positive file. 
  }
\description{
  A data frame with 299 different well IDs in which the cells treated with siRNAs targeting to PTB.If users didn't have the build-in positive controls, choose the parameters -withoutsvm and the filename can use any name such as 'NA'.
  }
\usage{data("negGene")}
\format{
  A data frame with 299 different well IDs in which the cells treated with siRNAs targeting to PTB.
  \describe{
    A data frame with 299 different well IDs in which the cells treated with siRNAs targeting to PTB. These wells were served as positive control.
  }
}
\details{
  These wells were designed by the authors in the large-scale screen.
}

\examples{
  data(posGene)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FDRcutoff.R
\name{FDRcutoff}
\alias{FDRcutoff}
\title{Determine optimal cutoff thresholds based on Screen Strength analysis.}
\usage{
FDRcutoff(zetaData, negGene, posGene, nonExpGene, combine = FALSE)
}
\arguments{
\item{zetaData}{A data frame containing zeta scores calculated by the Zeta() function. Should have columns 'Zeta_D' and 'Zeta_I' representing decrease and increase direction scores, respectively.}

\item{negGene}{A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in zetaData.}

\item{posGene}{A data frame or matrix containing positive control gene/siRNA identifiers. The first column should contain gene/siRNA names that match the row names in zetaData.}

\item{nonExpGene}{A data frame or matrix containing non-expressed gene/siRNA identifiers. These genes are used to estimate the baseline false discovery rate. The first column should contain gene/siRNA names that match the row names in zetaData.}

\item{combine}{Logical. Whether to combine decrease and increase direction zeta scores. Default is FALSE. When TRUE, uses the sum of Zeta_D and Zeta_I; when FALSE, analyzes each direction separately.}
}
\value{
A list containing:
  \item{FDR_cutOff}{A data frame with 6 columns:
    \itemize{
      \item Cut_Off: Zeta score threshold
      \item aFDR: Apparent false discovery rate at this threshold
      \item SS: Screen Strength = 1 - (aFDR / bFDR)
      \item TotalHits: Total number of hits at this threshold
      \item Num_nonExp: Number of non-expressed genes among hits
      \item Type: Direction ("Decrease", "Increase", or "Combine")
    }
  }
  \item{plotList}{A list with two ggplot objects:
    \itemize{
      \item Zeta_type: Jitter plots showing zeta score distributions by gene type
      \item SS_cutOff: Screen Strength curves showing SS vs. zeta score threshold
    }
  }
}
\description{
This function calculates optimal cutoff thresholds for identifying significant hits in high-throughput screening data using Screen Strength (SS) analysis. It evaluates the trade-off between sensitivity and specificity by calculating the ratio of apparent FDR to baseline FDR across different zeta score thresholds.
}
\details{
The function performs the following analysis:
\enumerate{
  \item Categorizes genes into types: "Gene" (test genes), "Positive" (positive controls), "NS_mix" (negative controls), and "non_exp" (non-expressed genes)
  \item Calculates baseline FDR (bFDR) as the proportion of non-expressed genes in the entire dataset
  \item For each zeta score threshold, calculates apparent FDR (aFDR) as the proportion of non-expressed genes among hits
  \item Computes Screen Strength: SS = 1 - (aFDR / bFDR)
  \item Generates plots showing zeta score distributions and SS curves
}

Higher Screen Strength values indicate better separation between true hits and false positives. Users can select appropriate thresholds based on desired sensitivity/specificity trade-offs.
}
\examples{
data(nonExpGene)
data(negGene)
data(posGene)
data(ZseqList)
data(countMat)
ZscoreVal <- Zscore(countMat, negGene)
zetaData <- Zeta(ZscoreVal, ZseqList, SVM=FALSE)
cutoffval <- FDRcutoff(zetaData, negGene, posGene, nonExpGene, combine=TRUE)

}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{FDR}
\keyword{ZetaSuite}
\keyword{cutoff}
\keyword{screen}
\keyword{strength}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SVM.R
\name{SVM}
\alias{SVM}
\title{Generate SVM decision boundaries for positive and negative control separation.}
\usage{
SVM(ECdataList)
}
\arguments{
\item{ECdataList}{A list containing event coverage data from the EventCoverage() function. The list should contain:
\itemize{
  \item EC_N_D: Event coverage matrix for negative controls in decrease direction
  \item EC_P_D: Event coverage matrix for positive controls in decrease direction
  \item EC_N_I: Event coverage matrix for negative controls in increase direction
  \item EC_P_I: Event coverage matrix for positive controls in increase direction
  \item ZseqList: Z-score thresholds for both directions
}}
}
\value{
A list containing two data frames:
  \item{cutOffD}{A data frame with SVM decision boundary points for decrease direction. Each row contains a Z-score threshold and the corresponding event coverage threshold that separates positive and negative controls.}
  \item{cutOffI}{A data frame with SVM decision boundary points for increase direction. Each row contains a Z-score threshold and the corresponding event coverage threshold that separates positive and negative controls.}
}
\description{
This function constructs Support Vector Machine (SVM) models to find optimal decision boundaries that separate positive and negative control samples in event coverage space. It uses radial kernel SVM to create non-linear decision boundaries for both decrease and increase directions.
}
\details{
The function performs the following steps:
\enumerate{
  \item Prepares training data by combining positive and negative control event coverage data
  \item Trains separate SVM models for decrease and increase directions using radial kernel
  \item Uses pre-tuned hyperparameters (cost=20, gamma=3 for decrease; cost=50, gamma=2 for increase)
  \item Generates prediction grids across the Z-score and event coverage space
  \item Identifies decision boundary points where the SVM prediction changes from negative to positive
  \item Returns the optimal threshold points for each Z-score bin
}

The resulting SVM curves can be used for background correction in zeta score calculations to improve the accuracy of hit identification.
}
\examples{
data(countMat)
data(negGene)
data(posGene)
ZscoreVal <- Zscore(countMat, negGene)
ECdataList <- EventCoverage(ZscoreVal, negGene, posGene, binNum=10, combine=TRUE)
\donttest{SVM(ECdataList)}

}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{SVM}
\keyword{ZetaSuite}
\keyword{boundary}
\keyword{decision}
\keyword{machine}
\keyword{support}
\keyword{vector}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Zscore.R
\name{Zscore}
\alias{Zscore}
\title{Z-score normalization for high-throughput screening data.}
\usage{
Zscore(countMat, negGene)
}
\arguments{
\item{countMat}{A matrix of raw count data where rows represent genes/siRNAs and columns represent readouts/conditions. The matrix should have row names corresponding to gene/siRNA identifiers.}

\item{negGene}{A data frame or matrix containing negative control gene/siRNA identifiers. The first column should contain the gene/siRNA names that match the row names in countMat.}
}
\value{
A Z-score normalized matrix with the same dimensions as the input countMat (excluding the Type column added during processing). Each value represents how many standard deviations away from the negative control mean that particular gene/readout combination is.
}
\description{
This function performs Z-score normalization on high-throughput screening data using negative control samples as reference. The Z-score transformation standardizes the data by centering and scaling each column (readout) based on the mean and standard deviation of negative control samples.
}
\details{
The function performs Z-score normalization as follows:
\enumerate{
  \item Extracts negative control samples from the input matrix using the identifiers provided in negGene
  \item For each column (readout), calculates the mean and standard deviation using only the negative control samples
  \item Applies Z-score transformation: \eqn{Z_{ij} = (X_{ij} - \mu_{j}) / \sigma_{j}}
  where \eqn{X_{ij}} is the raw value for gene \eqn{i} in readout \eqn{j}, \eqn{\mu_{j}} is the mean of negative controls in readout \eqn{j}, and \eqn{\sigma_{j}} is the standard deviation of negative controls in readout \eqn{j}
}

This normalization allows for comparison across different readouts and identifies genes/siRNAs that show significant deviation from the negative control distribution.
}
\examples{
data(countMat)
data(negGene)
ZscoreVal <- Zscore(countMat, negGene)
ZscoreVal[1:5, 1:5]

}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{Z-score}
\keyword{ZetaSuite}
\keyword{normalization}

\name{SVMcurve}
\alias{SVMcurve}
\docType{data}
\title{
  The SVM curve lines in Zeta-plot. 
}
\description{
  The SVM curves were calculated from raw input matrix files. They were designed to maximally seperate the positive and negative genes. 
  }
\usage{data("SVMcurve")}
\format{
  A data frame with 24 rows and 4 features.
  \describe{
    A data frame with 24 rows and 4 features.The first column is the bins cut-offs for decresed direction. The second column is the values of percentage with different cut-offs in column 1.
    The third column is the bins cut-offs for increased direction. The fourth column is the values of percentage with different cut-offs in column 3.
  }
}
\details{
  This data frame is the generated by SVM.R.
}

\examples{
  data(SVMcurve)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZetaSuitSC.R
\name{ZetaSuitSC}
\alias{ZetaSuitSC}
\title{Calculate zeta score for single cell RNA-seq quality control.}
\usage{
ZetaSuitSC(countMatSC, binNum = 10, filter = TRUE)
}
\arguments{
\item{countMatSC}{A matrix of single-cell RNA-seq count data where rows represent cells and columns represent genes.}

\item{binNum}{The number of bins for zeta score calculation. Default is 10. The function creates expression thresholds from 0 to the 80th percentile of non-zero expression values, divided into binNum intervals.}

\item{filter}{Logical. Whether to filter out cells with total read counts less than 100. Default is TRUE. This helps remove extremely low-quality cells before analysis.}
}
\value{
A list containing:
  \item{zetaData}{A data frame with two columns: 'Cell' (cell identifiers) and 'Zeta' (calculated zeta scores)}
  \item{p_cutoff}{A ggplot object showing the distribution of log10-transformed zeta scores with fitted Gaussian mixture components and the determined cutoff threshold}
}
\description{
This function evaluates the quality of cells detected in single-cell RNA-seq data by calculating a zeta score for each cell. The zeta score is based on the distribution of gene expression across different expression thresholds. A cutoff value is automatically determined using a two-component Gaussian mixture model to separate high-quality cells from low-quality or damaged cells.
}
\details{
The function works as follows:
\enumerate{
  \item Filters cells based on total read count if filter=TRUE
  \item Samples a subset of cells and genes for computational efficiency
  \item Creates expression thresholds (bins) from 0 to the 80th percentile of non-zero expression values
  \item For each cell, counts how many genes exceed each threshold
  \item Calculates the zeta score as a weighted sum of these counts
  \item Fits a two-component Gaussian mixture model to log10-transformed zeta scores
  \item Determines an optimal cutoff to separate high-quality from low-quality cells
}
}
\examples{
data(countMatSC)
\donttest{zetaDataSC <- ZetaSuitSC(countMatSC, binNum=50, filter=TRUE)}

}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{ZetaSuite}
\keyword{cell}
\keyword{control}
\keyword{quality}
\keyword{single}

\name{countMat}
\alias{countMat}
\docType{data}
\title{
  Subsampled data from in-house HTS2 screening for global splicing regulators.
}
\description{
  A data frame with 1609 individual screened genes and 100 functional readouts. The data was generated from a siRNA screen for global splicing regulators. In this screen, we interrogated ~400 endogenous alternative splicing (AS) events by using an oligo ligation-based strategy to quantify 18,480 pools of siRNAs against annotated protein-coding genes in the human genome.
}
\usage{data("countMat")}
\format{
  A data frame with 1609 observations on the following 100 variables
  \describe{
    A data frame with 1609 observations on the following 100 maker variables.Each row represents gene with specific knocking-down siRNA pool, each column is an AS event. The values in the matrix are the processed foldchange values between included exons and skipping exons read counts. 
  }
}
\details{
  This data frame is the raw output data from large-scale screening.
}

\examples{
  data(countMat)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Zeta.R
\name{Zeta}
\alias{Zeta}
\title{Calculation of zeta and weighted zeta score.}
\usage{
Zeta(ZscoreVal, ZseqList, SVMcurve = NULL, SVM = FALSE)
}
\arguments{
\item{ZscoreVal}{A matrix of Z-scores where rows represent genes and columns represent readouts/conditions. This is typically the output from the Zscore() function.}

\item{ZseqList}{A list containing two vectors: 'Zseq_D' (decrease direction thresholds) and 'Zseq_I' (increase direction thresholds). These define the Z-score bins for calculating event coverage.}

\item{SVMcurve}{Optional. A matrix containing SVM curve data for decrease and increase directions. Required only when SVM=TRUE. The matrix should have 4 columns: Z-score and coverage for decrease direction (columns 1-2), and Z-score and coverage for increase direction (columns 3-4).}

\item{SVM}{Logical. Whether to use SVM curves for background correction. Default is FALSE. When TRUE, the function subtracts SVM-predicted background from the event coverage before calculating zeta scores.}
}
\value{
A data frame with two columns:
  \item{Zeta_D}{Zeta score for decrease direction (exon skipping events)}
  \item{Zeta_I}{Zeta score for increase direction (exon inclusion events)}

Each row corresponds to a gene, and the zeta scores represent the cumulative regulatory effect across all Z-score thresholds.
}
\description{
This function calculates zeta scores for genes based on their Z-score profiles across different thresholds. The zeta score quantifies the regulatory effect of gene knockdown on alternative splicing events by measuring the area under the curve of event coverage across Z-score thresholds.
}
\details{
The function calculates zeta scores as follows:
\enumerate{
  \item For each Z-score threshold, calculates the proportion of readouts that exceed (increase) or fall below (decrease) the threshold
  \item Computes the area under the event coverage curve using trapezoidal integration
  \item If SVM=TRUE, subtracts SVM-predicted background coverage before area calculation
  \item Returns separate scores for decrease (Zeta_D) and increase (Zeta_I) directions
}

Higher zeta scores indicate stronger regulatory effects on alternative splicing.
}
\examples{
data(ZseqList)
data(SVMcurve)
data(countMat)
data(negGene)
ZscoreVal <- Zscore(countMat, negGene)
zetaData <- Zeta(ZscoreVal, ZseqList, SVM=FALSE)

}
\author{
Yajing Hao, Shuyang Zhang, Junhui Li, Guofeng Zhao, Xiang-Dong Fu
}
\keyword{ZetaSuite}
\keyword{alternative}
\keyword{score}
\keyword{splicing}
\keyword{zeta}
